# -*- encoding: utf-8 -*-
import copy
from abjad.tools import indicatortools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import pitchtools
from abjad.tools.abctools import AbjadObject
from abjad.tools.pitcharraytools.PitchArrayCell import PitchArrayCell


class PitchArrayRow(AbjadObject):
    r'''A pitch array row.

    ::

        >>> array = pitcharraytools.PitchArray([[1, 2, 1], [2, 1, 1]])
        >>> array[0].cells[0].pitches.append(0)
        >>> array[0].cells[1].pitches.append(2)
        >>> array[1].cells[2].pitches.append(4)
        >>> print array
        [c'] [d'    ] [  ]
        [         ] [ ] [e']

    ::

        >>> array[0]
        PitchArrayRow(c', d' x2, x1)

    ::

        >>> array[0].cell_widths
        (1, 2, 1)

    ::

        >>> array[0].dimensions
        (1, 4)

    ::

        >>> array[0].pitches
        (NamedPitch("c'"), NamedPitch("d'"))

    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        [],
        )

    ### INITIALIZER ###

    def __init__(self, cells):
        self._parent_array = None
        self._pitch_range = pitchtools.PitchRange(None, None)
        self._cells = []
        self.extend(cells)

    ### SPECIAL METHODS ###

    def __add__(self, arg):
        r'''Concatenates `arg` to pitch array row.

        Returns new pitch array row.
        '''
        if not isinstance(arg, PitchArrayRow):
            message = 'must be pitch array row.'
            raise TypeError(message)
        self_copy = copy.copy(self)
        arg_copy = copy.copy(arg)
        new_row = PitchArrayRow([])
        new_row.extend(self_copy.cells)
        new_row.extend(arg_copy.cells)
        return new_row

    def __copy__(self):
        r'''Copies pitch array row.

        Returns new pitch array row.
        '''
        new_cells = []
        for cell in self.cells:
            new_cell = copy.copy(cell)
            new_cells.append(new_cell)
        return PitchArrayRow(new_cells)

    def __eq__(self, arg):
        r'''True when `arg` is a pitch array row with contents equal to that of
        this pitch array row. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, PitchArrayRow):
            for self_cell, arg_cell in zip(self.cells, arg.cells):
                if not self_cell.matches_cell(arg_cell):
                    return False
                return True
        return False

    def __getitem__(self, arg):
        r'''Gets pitch array cell `arg` from pitch array row.

        Returns pitch array cell.
        '''
        if isinstance(arg, int):
            if 0 <= arg < self.width:
                accumulated_width = 0
                for cell in self.cells:
                    total_width = accumulated_width + cell.width
                    if accumulated_width <= arg < total_width:
                        return cell
                    accumulated_width = total_width
            elif 0 < abs(arg) < self.width:
                accumulated_width = 0
                abs_arg = abs(arg)
                for cell in reversed(self.cells):
                    total_width = accumulated_width + cell.width
                    if accumulated_width < abs_arg <= total_width:
                        return cell
                    accumulated_width = total_width
            else:
                message = 'no such cell in row.'
                raise IndexError(message)
        elif isinstance(arg, slice):
            cells = []
            start, stop, step = arg.indices(self.width)
            for cell_index in range(start, stop, step):
                cell = self[cell_index]
                if len(cells) == 0:
                    cells.append(cell)
                else:
                    if cells[-1] is not cell:
                        cells.append(cell)
            cells = tuple(cells)
            return cells
        else:
            message = 'must be int or slice.'
            raise ValueError(message)

    def __getstate__(self):
        r'''Gets object state.
        '''
        return vars(self)

    def __iadd__(self, arg):
        r'''Adds `arg` to pitch array row in place.

        Returns pitch array row.
        '''
        if not isinstance(arg, PitchArrayRow):
            message = 'must be pitch array row.'
            raise TypeError(message)
        copy_arg = copy.copy(arg)
        self.extend(copy_arg.cells)
        return self

    def __len__(self):
        r'''Length of pitch array row.

        Defined equal to the width of pitch array row.

        Returns nonnegative integer.
        '''
        return self.width

    def __ne__(self, arg):
        r'''True when pitch array row does not equal `arg`. Otherwise false.

        Returns boolean.
        '''
        return not self == arg

    def __repr__(self):
        r'''Gets interpreter representation of pitch array row.

        Returns string.
        '''
        return '{}({})'.format(type(self).__name__, self._compact_summary)

    def __str__(self):
        r'''String representation of pitch array row.

        Returns string.
        '''
        result = [str(cell) for cell in self.cells]
        result = ' '.join(result)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_summary(self):
        len_self = len(self.cells)
        if not len_self:
            return ' '
        elif 0 < len_self <= 8:
            result = [
                cell._format_row_column_repr_string for cell in self.cells]
            return ', '.join(result)
        else:
            left = ', '.join(
                [x._format_row_column_repr_string for x in self.cells[:2]])
            right = ', '.join(
                [x._format_row_column_repr_string for x in self.cells[-2:]])
            number_in_middle = len_self - 4
            middle = ', ... [%s] ..., '% number_in_middle
            return left + middle + right

    @property
    def _format_contents_string(self):
        result = []
        for cell in self.cells:
            result.append(cell._format_row_column_repr_string)
        result = ', '.join(result)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def cell_tokens(self):
        r'''Cell tokens of pitch array row.

        Returns tuple.
        '''
        return tuple([cell.token for cell in self.cells])

    @property
    def cell_widths(self):
        r'''Cell widths of pitch array row.

        Returns tuple.
        '''
        return tuple([cell.width for cell in self.cells])

    @property
    def cells(self):
        r'''Cells of pitch array row.

        Returns tuple.
        '''
        return tuple(self._cells)

    @property
    def depth(self):
        r'''Depth of pitch array row.

        Defined equal to ``1``.

        Returns ``1``.
        '''
        return 1

    @property
    def dimensions(self):
        r'''Dimensions of pitch array row.

        Returns pair.
        '''
        return self.depth, self.width

    @property
    def is_defective(self):
        r'''True when width of pitch array row does not equal width of parent
        pitch array. Otherwise false.

        Returns boolean.
        '''
        if self.parent_array is not None:
            return not self.width == self.parent_array.width
        return False

    @property
    def is_in_range(self):
        r'''True when all pitches in pitch array row are in pitch range of
        pitch array row. Otherwise false.

        Returns boolean.
        '''
        return all(pitch in self.pitch_range for pitch in self.pitches)

    @property
    def parent_array(self):
        r'''Parent pitch array housing pitch array row.

        Returns pitch array or none.
        '''
        return self._parent_array

    @property
    def pitch_range(self):
        r'''Gets and set pitch range of pitch array row.

        Returns pitch range.
        '''
        return self._pitch_range

    @pitch_range.setter
    def pitch_range(self, arg):
        if not isinstance(arg, pitchtools.PitchRange):
            message = 'must be pitch range.'
            raise TypeError(message)
        self._pitch_range = arg

    @property
    def pitches(self):
        r'''Pitches in pitch array row.

        Returns tuple.
        '''
        pitches = []
        for cell in self.cells:
            pitches.extend(cell.pitches)
        return tuple(pitches)

    @property
    def row_index(self):
        r'''Row index of pitch array row in parent pitch array.

        Returns nonnegative integer.
        '''
        parent_array = self.parent_array
        if parent_array is not None:
            return parent_array._rows.index(self)
        message = 'row has no parent array.'
        raise IndexError(message)

    @property
    def weight(self):
        r'''Weight of pitch array row.

        Defined equal to sum of weights of pitch array cells in pitch array
        row.

        Returns nonnegative integer.
        '''
        return sum([cell.weight for cell in self.cells])

    @property
    def width(self):
        r'''Width of pitch array row.

        Defined equal to sum of widths of pitch array cells in pitch array row.

        Returns nonnegative integer.
        '''
        return sum([cell.width for cell in self.cells])

    ### PUBLIC METHODS ###

    def append(self, cell_token):
        r'''Appends `cell_token` to pitch array row.

        Returns none.
        '''
        cell = PitchArrayCell(cell_token)
        cell._parent_row = self
        self._cells.append(cell)

    def apply_pitches(self, pitch_tokens):
        r'''Applies `pitch_tokens` to pitch cells in pitch array row.

        Returns none.
        '''
        pitch_tokens = pitch_tokens[:]
        if pitch_tokens:
            for cell in self.cells:
                if cell.pitches:
                    cell.pitches = [pitch_tokens.pop(0)]
        else:
            self.empty_pitches()

    def copy_subrow(self, start=None, stop=None):
        r'''Copies subrow of pitch array row from `start` to `stop`.

        Returns new pitch array row.
        '''
        arg = slice(start, stop)
        start, stop, step = arg.indices(self.width)
        if not step == 1:
            message = 'step no implemented.'
            raise NotImplementedError(message)
        column_indices = set(range(start, stop, step))
        row = PitchArrayRow([])
        cells = self[arg]
        new_cells = []
        for cell in cells:
            if not cell in new_cells:
                trim = [
                    x for x in cell.column_indices if x not in column_indices]
                new_width = cell.width - len(trim)
                new_cell = copy.copy(cell)
                new_cell._width = new_width
                new_cells.append(new_cell)
        row.extend(new_cells)
        return row

    def empty_pitches(self):
        r'''Empties pitches in pitch array row.

        Returns none.
        '''
        for cell in self.cells:
            cell.pitches = []

    def extend(self, cell_tokens):
        r'''Extends `cell_tokens` against pitch array row.

        Returns none.
        '''
        for cell_token in cell_tokens:
            self.append(cell_token)

    def has_spanning_cell_over_index(self, i):
        r'''True when pitch array row has one or more cells spanning over index
        `i`. Otherwise false.

        Returns boolean.
        '''
        try:
            cell = self[i]
            return cell.column_indices[0] < i
        except IndexError:
            return False

    def index(self, cell):
        r'''Index of pitch array `cell` in pitch array row.

        Retunrs nonnegative integer.
        '''
        return self._cells.index(cell)

    def merge(self, cells):
        r'''Merges `cells`.

        Returns pitch array cell.
        '''
        column_indices = []
        pitches = []
        width = 0
        for cell in cells:
            if not isinstance(cell, PitchArrayCell):
                raise TypeError
            if not cell.parent_row is self:
                message = 'cells must belong to row.'
                raise ValueError(message)
            column_indices.extend(cell.column_indices)
            pitches.extend(cell.pitches)
            width += cell.width
        start = min(column_indices)
        stop = start + len(column_indices)
        strict_series = range(start, stop)
        if not column_indices == strict_series:
            message = 'cells must be contiguous.'
            raise ValueError(message)
        first_cell = cells[0]
        for cell in cells[1:]:
            self.remove(cell)
        first_cell._pitches = pitches
        first_cell._width = width
        return first_cell

    def pad_to_width(self, width):
        r'''Pads pitch array row to `width`.

        Returns none.
        '''
        self_width = self.width
        if width < self_width:
            message = 'pad width must not be less than row width.'
            raise ValueError(message)
        missing_width = width - self_width
        for i in range(missing_width):
            cell = PitchArrayCell()
            self.append(cell)

    def pop(self, cell_index):
        r'''Pops cell `cell_index` from pitch array row.

        Returns pitch array cell.
        '''
        cell = self.pop(cell_index)
        cell._parent_row = None
        return cell

    def remove(self, cell):
        r'''Removes `cell` form pitch array row.

        Returns none.
        '''
        for i, x in enumerate(self.cells):
            if x is cell:
                self._cells.pop(i)
                break
        cell._parent_row = None

    def to_measure(self, cell_duration_denominator=8):
        r'''Changes pitch array row to measure with time signature
        numerator equal to pitch array row width and
        time signature denominator equal to `cell_duration_denominator`.

        ::

            >>> array = pitcharraytools.PitchArray([
            ...     [1, (2, 1), ([-2, -1.5], 2)],
            ...     [(7, 2), (6, 1), 1]])

        ::

            >>> print array
            [  ] [d'] [bf bqf    ]
            [g'     ] [fs'   ] [ ]

        ::

            >>> measure = array.rows[0].to_measure()

        ..  doctest::

            >>> print format(measure)
            {
                \time 4/8
                r8
                d'8
                <bf bqf>4
            }

        Returns measure.
        '''
        time_signature = indicatortools.TimeSignature(
            (self.width, cell_duration_denominator))
        measure = scoretools.Measure(time_signature, [])
        basic_cell_duration = \
            durationtools.Duration(1, cell_duration_denominator)
        measure_pitches, measure_durations = [], []
        for cell in self.cells:
            cell_pitches = cell.pitches
            if not cell_pitches:
                measure_pitches.append(None)
            elif len(cell_pitches) == 1:
                measure_pitches.append(cell_pitches[0])
            else:
                measure_pitches.append(cell_pitches)
            measure_duration = cell.width * basic_cell_duration
            measure_durations.append(measure_duration)
        leaves = scoretools.make_leaves(measure_pitches, measure_durations)
        measure.extend(leaves)
        return measure

    def withdraw(self):
        r'''Withdraws pitch array row from parent pitch array.

        Returns pitch array row.
        '''
        if self.parent_array is not None:
            self.parent_array.remove_row(self)
        return self



# -*- encoding: utf-8 -*-
import functools
from abjad.tools.abctools.AbjadObject import AbjadObject


@functools.total_ordering
class OrdinalConstant(AbjadObject):
    r'''An ordinal constant.

    Initializes with `dimension`, `value` and `representation`:

    ::

        >>> Left = datastructuretools.OrdinalConstant('x', -1, 'Left')
        >>> Left
        Left

    ::

        >>> Right = datastructuretools.OrdinalConstant('x', 1, 'Right')
        >>> Right
        Right

    ::

        >>> Left < Right
        True

    Comparing like-dimensioned ordinal constants is allowed:

    ::

        >>> Up = datastructuretools.OrdinalConstant('y', 1, 'Up')
        >>> Up
        Up

    ::

        >>> Down = datastructuretools.OrdinalConstant('y', -1, 'Down')
        >>> Down
        Down

    ::

        >>> Down < Up
        True

    Comparing differently dimensioned ordinal constants raises an exception:

    ::

        >>> import pytest

    ::

        >>> bool(pytest.raises(Exception, 'Left < Up'))
        True

    The ``Left``, ``Right``, ``Center``, ``Up`` and ``Down`` constants 
    shown here load into Python's built-in namespace on Abjad import.

    These four objects can be used as constant values supplied to keywords.

    This behavior is similar to True, False and None.

    Ordinal constants are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_dimension',
        '_representation',
        '_value',
        )

    ### CONSTRUCTOR ###

    def __new__(
        cls, 
        dimension=None, 
        value=0, 
        representation=None,
        ):
        dimension = dimension or ''
        representation = representation or ''
        assert isinstance(dimension, str), repr(dimension)
        assert isinstance(representation, str), repr(representation)
        self = object.__new__(cls)
        self._dimension = dimension
        self._value = value
        self._representation = representation
        return self

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is an ordinal constant with dimension and value
        equal to those of this ordinal constant. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self._dimension == expr._dimension:
                return self._value == expr._value
        return False

    def __format__(self, format_specification=''):
        r'''Formats ordinal constant.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (
            self._dimension,
            self._value,
            self._representation,
            )

    def __lt__(self, expr):
        r'''True when `expr` is an ordinal with value greater than that of this
        ordinal constant. Otherwise false.

        Returns boolean.
        '''
        self._check_comparator(expr)
        return self._value < expr._value

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            storage_format_pieces=(
                self._representation,
                ),
            )

    ### PRIVATE METHODS ###

    # can only compare like-dimensioned ordinal constants
    def _check_comparator(self, expr):
        if not isinstance(expr, type(self)) or \
            not self._dimension == expr._dimension:
            message = 'can only compare like-dimensioned ordinal constants.'
            raise Exception(message)



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_negate_sequence_elements_cyclically_01():

    sequence_1 = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    sequence_2 = sequencetools.negate_sequence_elements_cyclically(sequence_1, [0, 1, 2], 5)

    assert sequence_2 == [-1, -2, -3, 4, 5, -6, -7, -8, 9, 10]



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Container_is_simultaneous_01():
    r'''True when container encloses contents in LilyPond << >> brackets,
    otherwise False.
    '''

    assert not Container([]).is_simultaneous
    assert not scoretools.FixedDurationTuplet(Duration(2, 8), []).is_simultaneous
    assert not Tuplet(Multiplier(2, 3), []).is_simultaneous
    assert scoretools.GrandStaff([]).is_simultaneous
    assert not scoretools.make_rhythmic_sketch_staff([]).is_simultaneous
    assert not scoretools.RhythmicStaff([]).is_simultaneous
    assert not Measure((4, 8), []).is_simultaneous
    assert Score([]).is_simultaneous
    assert not Container([]).is_simultaneous
    assert not Staff([]).is_simultaneous
    assert scoretools.StaffGroup([]).is_simultaneous
    assert not Voice([]).is_simultaneous


def test_scoretools_Container_is_simultaneous_02():
    r'''True when container encloses contents in LilyPond << >> brackets,
    otherwise False.
    '''

    container = Container([])
    container.is_simultaneous = True
    assert container.is_simultaneous


def test_scoretools_Container_is_simultaneous_03():
    r'''Container 'simultaneous' is settable.
    '''

    container = Container([])
    assert not container.is_simultaneous

    container.is_simultaneous = True
    assert container.is_simultaneous


def test_scoretools_Container_is_simultaneous_04():
    r'''A simultaneous container can hold Contexts.
    '''

    container = Container([Voice("c'8 cs'8"), Voice("d'8 ef'8")])
    container.is_simultaneous = True

    assert systemtools.TestManager.compare(
        container,
        r'''
        <<
            \new Voice {
                c'8
                cs'8
            }
            \new Voice {
                d'8
                ef'8
            }
        >>
        '''
        )

    r'''
    <<
        \new Voice {
            c'8
            cs'8
        }
        \new Voice {
            d'8
            ef'8
        }
    >>
    '''


# simultaneous Errors #

def test_scoretools_Container_is_simultaneous_05():
    r'''simultaneous containers must contain only Contexts.
    It can not take leaves.
    '''

    container = Container(scoretools.make_repeated_notes(4))
    pytest.raises(AssertionError, 'container.is_simultaneous = True')


def test_scoretools_Container_is_simultaneous_06():
    r'''simultaneous containers must contain only Contexts.
    It can not take Containers.
    '''

    container = Container(Container(scoretools.make_repeated_notes(4)) * 2)
    pytest.raises(AssertionError, 'container.is_simultaneous = True')



# -*- encoding: utf-8 -*-
import fractions
import math
import re
from abjad.tools import mathtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class Duration(AbjadObject, fractions.Fraction):
    r'''A duration.

    ..  container:: example

        Initializes from integer numerator:

        ::

            >>> Duration(3)
            Duration(3, 1)

    ..  container:: example

        Initializes from integer numerator and denominator:

        ::

            >>> Duration(3, 16)
            Duration(3, 16)

    ..  container:: example

        Initializes from integer-equivalent numeric numerator:

        ::

            >>> Duration(3.0)
            Duration(3, 1)

    ..  container:: example

        Initializes from integer-equivalent numeric numerator and denominator:

        ::

            >>> Duration(3.0, 16)
            Duration(3, 16)

    ..  container:: example

        Initializes from integer-equivalent singleton:

        ::

            >>> Duration((3,))
            Duration(3, 1)

    ..  container:: example

        Initializes from integer-equivalent pair:

        ::

            >>> Duration((3, 16))
            Duration(3, 16)

    ..  container:: example

        Initializes from other duration:

        ::

            >>> Duration(Duration(3, 16))
            Duration(3, 16)

    ..  container:: example

        Intialize from fraction:

        ::

            >>> Duration(Fraction(3, 16))
            Duration(3, 16)

    ..  container:: example

        Initializes from solidus string:

        ::

            >>> Duration('3/16')
            Duration(3, 16)

    ..  container:: example

        Initializes from nonreduced fraction:

        ::

            >>> Duration(mathtools.NonreducedFraction(3, 16))
            Duration(3, 16)

    ..  container:: example

        Durations inherit from built-in fraction:

        ::

            >>> isinstance(Duration(3, 16), Fraction)
            True

    ..  container:: example

        Durations are numeric:

        ::

            >>> import numbers

        ::

            >>> isinstance(Duration(3, 16), numbers.Number)
            True

    '''

    ### CONSTRUCTOR ###

    def __new__(cls, *args):
        from abjad.tools import sequencetools
        try:
            return fractions.Fraction.__new__(cls, *args)
        except TypeError:
            pass
        try:
            return fractions.Fraction.__new__(cls, *args[0])
        except (AttributeError, TypeError):
            pass
        try:
            return fractions.Fraction.__new__(
                cls, args[0].numerator, args[0].denominator)
        except AttributeError:
            pass
        if len(args) == 1 and \
            sequencetools.is_integer_equivalent_singleton(args[0]):
            self = fractions.Fraction.__new__(cls, int(args[0][0]))
        elif len(args) == 1 and \
            sequencetools.is_fraction_equivalent_pair(args[0]):
            self = fractions.Fraction.__new__(
                cls, int(args[0][0]), int(args[0][1]))
        elif len(args) == 1 and \
            isinstance(args[0], str) and not '/' in args[0]:
            result = Duration._initialize_from_lilypond_duration_string(args[0])
            self = fractions.Fraction.__new__(cls, result)
        elif sequencetools.all_are_integer_equivalent_numbers(args):
            self = fractions.Fraction.__new__(cls, *[int(x) for x in args])
        else:
            raise ValueError(args)
        return self

    ### SPECIAL METHODS ###

    def __abs__(self, *args):
        r'''Absolute value of duration.

        Returns nonnegative duration.
        '''
        return type(self)(fractions.Fraction.__abs__(self, *args))

    def __add__(self, *args):
        r'''Adds duration to `args`.

        Returns duration when `args` is a duration:

        ::

            >>> duration_1 = Duration(1, 2)
            >>> duration_2 = Duration(3, 2)
            >>> duration_1 + duration_2
            Duration(2, 1)

        Returns nonreduced fraction when `args` is a nonreduced fraction:

        ::

            >>> duration = Duration(1, 2)
            >>> nonreduced_fraction = mathtools.NonreducedFraction(3, 6)
            >>> duration + nonreduced_fraction
            NonreducedFraction(6, 6)

        Returns duration.
        '''
        if len(args) == 1 and \
            isinstance(args[0], mathtools.NonreducedFraction):
            result = args[0].__radd__(self)
        else:
            result = type(self)(fractions.Fraction.__add__(self, *args))
        return result

    def __div__(self, *args):
        r'''Divides duration by `args`.

        Returns multiplier.
        '''
        from abjad.tools import durationtools
        if len(args) == 1 and isinstance(args[0], type(self)):
            fraction = fractions.Fraction.__div__(self, *args)
            result = durationtools.Multiplier(fraction)
        elif len(args) == 1 and isinstance(
            args[0], mathtools.NonreducedFraction):
            result =  args[0].__rdiv__(self)
        else:
            result = type(self)(fractions.Fraction.__div__(self, *args))
        return result

    def __divmod__(self, *args):
        r'''Equals the pair (duration // `args`, duration % `args`).

        Returns pair.
        '''
        truncated, residue = fractions.Fraction.__divmod__(self, *args)
        truncated = type(self)(truncated)
        residue = type(self)(residue)
        return truncated, residue

    def __eq__(self, arg):
        r'''True when duration equals `arg`.
        Otherwise false.

        Returns boolean.
        '''
        return fractions.Fraction.__eq__(self, arg)

    def __format__(self, format_specification=''):
        r'''Formats duration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __ge__(self, arg):
        r'''True when duration is greater than or equal to `arg`.
        Otherwise false.

        Returns boolean.
        '''
        return fractions.Fraction.__ge__(self, arg)

    def __gt__(self, arg):
        r'''True when duration is greater than `arg`.
        Otherwise false.

        Returns boolean.
        '''
        return fractions.Fraction.__gt__(self, arg)

    def __le__(self, arg):
        r'''True when duration is less than or equal to `arg`.
        Otherwise false.

        Returns boolean.
        '''
        return fractions.Fraction.__le__(self, arg)

    def __lt__(self, arg):
        r'''True when duration is less than `arg`.
        Otherwise false.

        Returns boolean.
        '''
        return fractions.Fraction.__lt__(self, arg)

    def __mod__(self, *args):
        r'''Modulus operator applied to duration.

        Returns duration.
        '''
        return type(self)(fractions.Fraction.__mod__(self, *args))

    def __mul__(self, *args):
        r'''Duration multiplied by `args`.

        Returns a new duration when `args` is a duration:

        ::

            >>> duration_1 = Duration(1, 2)
            >>> duration_2 = Duration(3, 2)
            >>> duration_1 * duration_2
            Duration(3, 4)

        Returns nonreduced fraction when `args` is a nonreduced fraction:

        ::

            >>> duration = Duration(1, 2)
            >>> nonreduced_fraction = mathtools.NonreducedFraction(3, 6)
            >>> duration * nonreduced_fraction
            NonreducedFraction(3, 12)

        Returns duration or nonreduced fraction.
        '''
        if len(args) == 1 and \
            isinstance(args[0], mathtools.NonreducedFraction):
            result = args[0].__rmul__(self)
        else:
            result = type(self)(fractions.Fraction.__mul__(self, *args))
        return result

    def __ne__(self, arg):
        r'''True when duration does not equal `arg`.
        Otherwise false.

        Returns boolean.
        '''
        return fractions.Fraction.__ne__(self, arg)

    def __neg__(self, *args):
        r'''Negation of duration.

        Returns duration.
        '''
        return type(self)(fractions.Fraction.__neg__(self, *args))

    def __pos__(self, *args):
        r'''Positive duration.

        Returns duration.
        '''
        return type(self)(fractions.Fraction.__pos__(self, *args))

    def __pow__(self, *args):
        r'''Duration raised to `args` power.

        Returns duration.
        '''
        return type(self)(fractions.Fraction.__pow__(self, *args))

    def __radd__(self, *args):
        r'''Adds `args` to duration.

        Returns duration.
        '''
        return type(self)(fractions.Fraction.__radd__(self, *args))

    def __rdiv__(self, *args):
        r'''Divides `args` by duration.

        Returns duration.
        '''
        return type(self)(fractions.Fraction.__rdiv__(self, *args))

    def __rdivmod__(self, *args):
        r'''Documentation required.
        '''
        return type(self)(fractions.Fraction.__rdivmod__(self, *args))

    def __reduce__(self):
        r'''Documentation required.
        '''
        return type(self), (self.numerator, self.denominator)

    def __reduce_ex__(self, protocol):
        r'''Documentation required.
        '''
        return type(self), (self.numerator, self.denominator)

    def __rmod__(self, *args):
        r'''Documentation required.
        '''
        return type(self)(fractions.Fraction.__rmod__(self, *args))

    def __rmul__(self, *args):
        r'''Multiplies `args` by duration.

        Returns new duration.
        '''
        return type(self)(fractions.Fraction.__rmul__(self, *args))

    def __rpow__(self, *args):
        r'''Raises `args` to the power of duration.

        Returns new duration.
        '''
        return type(self)(fractions.Fraction.__rpow__(self, *args))

    def __rsub__(self, *args):
        r'''Subtracts duration from `args`.

        Returns new duration.
        '''
        return type(self)(fractions.Fraction.__rsub__(self, *args))

    def __rtruediv__(self, *args):
        r'''Documentation required.

        Returns new duration.
        '''
        return type(self)(fractions.Fraction.__rtruediv__(self, *args))

    def __sub__(self, *args):
        r'''Subtracts `args` from duration.

        Returns new duration.
        '''
        if len(args) == 1 and \
            isinstance(args[0], mathtools.NonreducedFraction):
            return args[0].__rsub__(self)
        else:
            return type(self)(fractions.Fraction.__sub__(self, *args))

    def __truediv__(self, *args):
        r'''Documentation required.
        '''
        return type(self)(fractions.Fraction.__truediv__(self, *args))

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                self.numerator,
                self.denominator,
                ),
            )

    ### PRIVATE METHODS ###

    @staticmethod
    def _group_nonreduced_fractions_by_implied_prolation(durations):
        durations = [
            mathtools.NonreducedFraction(duration)
            for duration in durations
            ]
        assert 0 < len(durations)
        group = [durations[0]]
        result = [group]
        for d in durations[1:]:
            d_f = set(mathtools.factors(d.denominator))
            d_f.discard(2)
            gd_f = set(mathtools.factors(group[0].denominator))
            gd_f.discard(2)
            if d_f == gd_f:
                group.append(d)
            else:
                group = [d]
                result.append(group)
        return result

    @staticmethod
    def _initialize_from_lilypond_duration_string(duration_string):
        numeric_body_strings = [str(2 ** n) for n in range(8)]
        other_body_strings = [r'\\breve', r'\\longa', r'\\maxima']
        body_strings = numeric_body_strings + other_body_strings
        body_strings = '|'.join(body_strings)
        pattern = r'^(%s)(\.*)$' % body_strings
        match = re.match(pattern, duration_string)
        if match is None:
            message = 'incorrect duration string format: {!r}.'
            message = message.format(duration_string)
            raise TypeError(message)
        body_string, dots_string = match.groups()
        try:
            body_denominator = int(body_string)
            body_duration = fractions.Fraction(1, body_denominator)
        except ValueError:
            if body_string == r'\breve':
                body_duration = fractions.Fraction(2)
            elif body_string == r'\longa':
                body_duration = fractions.Fraction(4)
            elif body_string == r'\maxima':
                body_duration = fractions.Fraction(8)
            else:
                message = 'unknown body string: {!r}.'
                message = message.format(body_string)
                raise ValueError(message)
        rational = body_duration
        for n in range(len(dots_string)):
            exponent = n + 1
            denominator = 2 ** exponent
            multiplier = fractions.Fraction(1, denominator)
            addend = multiplier * body_duration
            rational += addend
        return rational

    ### PUBLIC PROPERTIES ###

    @property
    def dot_count(self):
        r'''Number of dots required to notate duration.

        ::

            >>> for n in range(1, 16 + 1):
            ...     try:
            ...         duration = Duration(n, 16)
            ...         sixteenths = duration.with_denominator(16)
            ...         dot_count = duration.dot_count
            ...         string = '{!s}\t{}'
            ...         string = string.format(sixteenths, dot_count)
            ...         print string
            ...     except AssignabilityError:
            ...         sixteenths = duration.with_denominator(16)
            ...         string = '{!s}\t{}'
            ...         string = string.format(sixteenths, '--')
            ...         print string
            ...
            1/16    0
            2/16    0
            3/16    1
            4/16    0
            5/16    --
            6/16    1
            7/16    2
            8/16    0
            9/16    --
            10/16   --
            11/16   --
            12/16   1
            13/16   --
            14/16   2
            15/16   3
            16/16   0

        Raises assignability error when duration is not assignable.

        Returns positive integer.
        '''
        if not self.is_assignable:
            raise AssignabilityError
        binary_string = mathtools.integer_to_binary_string(self.numerator)
        digit_sum = sum([int(x) for x in list(binary_string)])
        dot_count = digit_sum - 1
        return dot_count

    @property
    def equal_or_greater_assignable(self):
        r'''Assignable duration equal to or just greater than this duration.

        ::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_greater_assignable
            ...     sixteenths = duration.with_denominator(16)
            ...     print '{!s}\t{!s}'.format(sixteenths, result)
            ...
            1/16    1/16
            2/16    1/8
            3/16    3/16
            4/16    1/4
            5/16    3/8
            6/16    3/8
            7/16    7/16
            8/16    1/2
            9/16    3/4
            10/16   3/4
            11/16   3/4
            12/16   3/4
            13/16   7/8
            14/16   7/8
            15/16   15/16
            16/16   1

        Returns new duration.
        '''
        good_denominator = mathtools.greatest_power_of_two_less_equal(
            self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator += 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_greater_power_of_two(self):
        r'''Duration equal or just greater power of ``2``.

        ::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_greater_power_of_two
            ...     sixteenths = duration.with_denominator(16)
            ...     print '{!s}\t{!s}'.format(sixteenths, result)
            ...
            1/16    1/16
            2/16    1/8
            3/16    1/4
            4/16    1/4
            5/16    1/2
            6/16    1/2
            7/16    1/2
            8/16    1/2
            9/16    1
            10/16   1
            11/16   1
            12/16   1
            13/16   1
            14/16   1
            15/16   1
            16/16   1

        Returns new duration.
        '''
        denominator_exponent = -int(math.ceil(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent

    @property
    def equal_or_lesser_assignable(self):
        r'''Assignable duration equal or just less than this duration.

        ::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_lesser_assignable
            ...     sixteenths = duration.with_denominator(16)
            ...     print '{!s}\t{!s}'.format(sixteenths, result)
            ...
            1/16    1/16
            2/16    1/8
            3/16    3/16
            4/16    1/4
            5/16    1/4
            6/16    3/8
            7/16    7/16
            8/16    1/2
            9/16    1/2
            10/16   1/2
            11/16   1/2
            12/16   3/4
            13/16   3/4
            14/16   7/8
            15/16   15/16
            16/16   1

        Returns new duration.
        '''
        good_denominator = mathtools.least_power_of_two_greater_equal(
            self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator -= 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_lesser_power_of_two(self):
        r'''Duration of the form ``d**2`` equal to or just less than this
        duration.

        ::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_lesser_power_of_two
            ...     sixteenths = duration.with_denominator(16)
            ...     print '{!s}\t{!s}'.format(sixteenths, result)
            ...
            1/16    1/16
            2/16    1/8
            3/16    1/8
            4/16    1/4
            5/16    1/4
            6/16    1/4
            7/16    1/4
            8/16    1/2
            9/16    1/2
            10/16   1/2
            11/16   1/2
            12/16   1/2
            13/16   1/2
            14/16   1/2
            15/16   1/2
            16/16   1

        Returns new duration.
        '''
        denominator_exponent = -int(math.floor(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent

    @property
    def flag_count(self):
        r'''Number of flags required to notate duration.

        ::

            >>> for n in range(1, 16 + 1):
            ...     duration = Duration(n, 64)
            ...     sixty_fourths = duration.with_denominator(64)
            ...     print '{!s}\t{}'.format(sixty_fourths, duration.flag_count)
            ...
            1/64    4
            2/64    3
            3/64    3
            4/64    2
            5/64    2
            6/64    2
            7/64    2
            8/64    1
            9/64    1
            10/64   1
            11/64   1
            12/64   1
            13/64   1
            14/64   1
            15/64   1
            16/64   0

        Returns nonnegative integer.
        '''
        # TODO: rewrite with only one operation per line
        flag_count = max(-int(math.floor(math.log(float(self.numerator) /
            self.denominator, 2))) - 2, 0)
        return flag_count

    @property
    def has_power_of_two_denominator(self):
        r'''True when duration is an integer power of ``2``.
        Otherwise false:

        ::

            >>> for n in range(1, 16 + 1):
            ...     duration = Duration(1, n)
            ...     result = duration.has_power_of_two_denominator
            ...     print '{!s}\t{}'.format(duration, result)
            ...
            1       True
            1/2     True
            1/3     False
            1/4     True
            1/5     False
            1/6     False
            1/7     False
            1/8     True
            1/9     False
            1/10    False
            1/11    False
            1/12    False
            1/13    False
            1/14    False
            1/15    False
            1/16    True

        Returns boolean.
        '''
        exponent = math.log(self.denominator, 2)
        return int(exponent) == exponent

    @property
    def implied_prolation(self):
        r'''Implied prolation of duration.

        ::

            >>> for denominator in range(1, 16 + 1):
            ...     duration = Duration(1, denominator)
            ...     result = duration.implied_prolation
            ...     print '{!s}\t{!s}'.format(duration, result)
            ...
            1       1
            1/2     1
            1/3     2/3
            1/4     1
            1/5     4/5
            1/6     2/3
            1/7     4/7
            1/8     1
            1/9     8/9
            1/10    4/5
            1/11    8/11
            1/12    2/3
            1/13    8/13
            1/14    4/7
            1/15    8/15
            1/16    1

        Returns new multipler.
        '''
        from abjad.tools import durationtools
        numerator = \
            mathtools.greatest_power_of_two_less_equal(self.denominator)
        return durationtools.Multiplier(numerator, self.denominator)

    @property
    def is_assignable(self):
        r'''True when duration is assignable. Otherwise false:

        ::

            >>> for numerator in range(0, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     sixteenths = duration.with_denominator(16)
            ...     print '{!s}\t{}'.format(sixteenths, duration.is_assignable)
            ...
            0/16    False
            1/16    True
            2/16    True
            3/16    True
            4/16    True
            5/16    False
            6/16    True
            7/16    True
            8/16    True
            9/16    False
            10/16   False
            11/16   False
            12/16   True
            13/16   False
            14/16   True
            15/16   True
            16/16   True

        Returns boolean.
        '''
        if 0 < self < 16:
            if mathtools.is_nonnegative_integer_power_of_two(
                self.denominator):
                if mathtools.is_assignable_integer(self.numerator):
                    return True
        return False

    @property
    def lilypond_duration_string(self):
        r'''LilyPond duration string of duration.

            >>> Duration(3, 16).lilypond_duration_string
            '8.'

        Raises assignability error when duration is not assignable.

        Returns string.
        '''
        from abjad.tools import durationtools

        if not self.is_assignable:
            raise AssignabilityError(self)
        undotted_rational = self.equal_or_lesser_power_of_two
        if undotted_rational <= 1:
            undotted_duration_string = str(undotted_rational.denominator)
        elif undotted_rational == type(self)(2, 1):
            undotted_duration_string = r'\breve'
        elif undotted_rational == type(self)(4, 1):
            undotted_duration_string = r'\longa'
        elif undotted_rational == type(self)(8, 1):
            undotted_duration_string = r'\maxima'
        else:
            message = 'can not process undotted rational: {}'
            message = message.format(undotted_rational)
            raise ValueError(message)
        dot_count = self.dot_count
        dot_string = '.' * dot_count
        dotted_duration_string = undotted_duration_string + dot_string
        return dotted_duration_string

    @property
    def pair(self):
        '''Duration numerator and denominator.

        ::

            >>> duration = Duration(3, 16)

        ::

            >>> duration.pair
            (3, 16)

        Returns integer pair.
        '''
        return self.numerator, self.denominator

    @property
    def prolation_string(self):
        r'''Prolation string of duration.

        ::

            >>> generator = Duration.yield_durations(unique=True)
            >>> for n in range(16):
            ...     duration = generator.next()
            ...     string = '{!s}\t{}'
            ...     string = string.format(duration, duration.prolation_string)
            ...     print string
            ...
            1       1:1
            2       1:2
            1/2     2:1
            1/3     3:1
            3       1:3
            4       1:4
            3/2     2:3
            2/3     3:2
            1/4     4:1
            1/5     5:1
            5       1:5
            6       1:6
            5/2     2:5
            4/3     3:4
            3/4     4:3
            2/5     5:2

        Returns string.
        '''
        return '{}:{}'.format(self.denominator, self.numerator)

    @property
    def reciprocal(self):
        '''Reciprocal of duration.

        Returns new duration.
        '''
        return type(self)(self.denominator, self.numerator)

    ### PUBLIC FUNCTIONS ###

    @staticmethod
    def durations_to_nonreduced_fractions(durations):
        r'''Change `durations` to nonreduced fractions sharing
        least common denominator.

        ::

            >>> durations = [Duration(2, 4), 3, (5, 16)]
            >>> result = Duration.durations_to_nonreduced_fractions(durations)
            >>> for x in result:
            ...     x
            ...
            NonreducedFraction(8, 16)
            NonreducedFraction(48, 16)
            NonreducedFraction(5, 16)

        Returns new object of `durations` type.
        '''
        durations = [Duration(x) for x in durations]
        denominators = [duration.denominator for duration in durations]
        lcd = mathtools.least_common_multiple(*denominators)
        nonreduced_fractions = [
            mathtools.NonreducedFraction(x).with_denominator(lcd)
            for x in durations
            ]
        result = type(durations)(nonreduced_fractions)
        return result

    @staticmethod
    def from_lilypond_duration_string(lilypond_duration_string):
        r'''Initializes duration from LilyPond duration string.

        ::

            >>> Duration.from_lilypond_duration_string('8.')
            Duration(3, 16)

        Returns duration.
        '''
        fraction = Duration._initialize_from_lilypond_duration_string(
            lilypond_duration_string)
        return Duration(fraction)

    @staticmethod
    def is_token(expr):
        '''True when `expr` correctly initializes a duration.
        Otherwise false:

        ::

            >>> Duration.is_token('8.')
            True

        Returns boolean.
        '''
        try:
            Duration.__new__(Duration, expr)
            return True
        except:
            return False

    def to_clock_string(self, escape_ticks=False):
        r'''Changes duration to clock string.

        ..  container:: example

            Changes numeric `seconds` to clock string:

            ::

                >>> duration = Duration(117)
                >>> duration.to_clock_string()
                '1\'57"'

        ..  container:: example

            Changes numeric `seconds` to escaped clock string:

            ::

                >>> note = Note("c'4")
                >>> clock_string = duration.to_clock_string(escape_ticks=True)

            ::

                >>> markup = markuptools.Markup('"%s"' % clock_string, Up)
                >>> attach(markup, note)

            ..  doctest::

                >>> print format(note)
                c'4 ^ \markup { 1'57\\" }

        Returns string.
        '''
        if self < 0:
            message = 'seconds must be positive: {!r}.'
            message = message.format(self)
            raise ValueError(message)
        minutes = int(self / 60)
        remaining_seconds = str(int(self - minutes * 60)).zfill(2)
        if escape_ticks:
            clock_string = "{}'{}\\\"".format(minutes, remaining_seconds)
        else:
            clock_string = "{}'{}\"".format(minutes, remaining_seconds)
        return clock_string

    def with_denominator(self, denominator):
        r'''Change this duration to new duration with `denominator`.

        ::

            >>> duration = Duration(1, 4)
            >>> for denominator in (4, 8, 16, 32):
            ...     print duration.with_denominator(denominator)
            ...
            1/4
            2/8
            4/16
            8/32

        Returns new duration.
        '''
        nonreduced_fraction = mathtools.NonreducedFraction(self)
        return nonreduced_fraction.with_denominator(denominator)

    @staticmethod
    def yield_durations(unique=False):
        r'''Yields all positive durations.

        ..  container:: example

            Yields all positive durations in Cantor diagonalized order:

            ::

                >>> generator = Duration.yield_durations()
                >>> for n in range(16):
                ...     generator.next()
                ...
                Duration(1, 1)
                Duration(2, 1)
                Duration(1, 2)
                Duration(1, 3)
                Duration(1, 1)
                Duration(3, 1)
                Duration(4, 1)
                Duration(3, 2)
                Duration(2, 3)
                Duration(1, 4)
                Duration(1, 5)
                Duration(1, 2)
                Duration(1, 1)
                Duration(2, 1)
                Duration(5, 1)
                Duration(6, 1)

        ..  container:: example

            Yields all positive durations in Cantor diagonalized order
            uniquely:

            ::

                >>> generator = Duration.yield_durations(unique=True)
                >>> for n in range(16):
                ...     generator.next()
                ...
                Duration(1, 1)
                Duration(2, 1)
                Duration(1, 2)
                Duration(1, 3)
                Duration(3, 1)
                Duration(4, 1)
                Duration(3, 2)
                Duration(2, 3)
                Duration(1, 4)
                Duration(1, 5)
                Duration(5, 1)
                Duration(6, 1)
                Duration(5, 2)
                Duration(4, 3)
                Duration(3, 4)
                Duration(2, 5)

        Returns generator.
        '''
        generator = mathtools.yield_nonreduced_fractions()
        while True:
            integer_pair = generator.next()
            duration = Duration(integer_pair)
            if not unique:
                yield duration
            elif duration.pair == integer_pair:
                yield duration

    def yield_equivalent_durations(self, minimum_written_duration=None):
        r'''Yields all durations equivalent to this duration.

        Returns output in Cantor diagonalized order.

        Ensures written duration never less than `minimum_written_duration`.

        ..  container:: example

            Yields durations equivalent to ``1/8``:

            ::

                >>> pairs = Duration(1, 8).yield_equivalent_durations()
                >>> for pair in pairs: pair
                ...
                (Multiplier(1, 1), Duration(1, 8))
                (Multiplier(2, 3), Duration(3, 16))
                (Multiplier(4, 3), Duration(3, 32))
                (Multiplier(4, 7), Duration(7, 32))
                (Multiplier(8, 7), Duration(7, 64))
                (Multiplier(8, 15), Duration(15, 64))
                (Multiplier(16, 15), Duration(15, 128))
                (Multiplier(16, 31), Duration(31, 128))

        ..  container:: example

            Yields durations equivalent ot ``1/12``:

            ::

                >>> pairs = Duration(1, 12).yield_equivalent_durations()
                >>> for pair in pairs: pair
                ...
                (Multiplier(2, 3), Duration(1, 8))
                (Multiplier(4, 3), Duration(1, 16))
                (Multiplier(8, 9), Duration(3, 32))
                (Multiplier(16, 9), Duration(3, 64))
                (Multiplier(16, 21), Duration(7, 64))
                (Multiplier(32, 21), Duration(7, 128))
                (Multiplier(32, 45), Duration(15, 128))

        ..  container:: example

            Yields durations equivalent to ``5/48``:

            ::

                >>> pairs = Duration(5, 48).yield_equivalent_durations()
                >>> for pair in pairs: pair
                ...
                (Multiplier(5, 6), Duration(1, 8))
                (Multiplier(5, 3), Duration(1, 16))
                (Multiplier(5, 9), Duration(3, 16))
                (Multiplier(10, 9), Duration(3, 32))
                (Multiplier(20, 21), Duration(7, 64))
                (Multiplier(40, 21), Duration(7, 128))
                (Multiplier(8, 9), Duration(15, 128))

        Defaults `minimum_written_duration` to ``1/128``.

        Returns generator.
        '''
        if minimum_written_duration is None:
            minimum_written_duration = type(self)(1, 128)
        else:
            minimum_written_duration = type(self)(minimum_written_duration)
        generator = type(self).yield_durations(unique=True)
        pairs = []
        while True:
            written_duration = generator.next()
            if not written_duration.is_assignable:
                continue
            if written_duration < minimum_written_duration:
                pairs = tuple(pairs)
                return pairs
            prolation = self / written_duration
            if prolation.is_proper_tuplet_multiplier:
                pair = (prolation, written_duration)
                pairs.append(pair)



# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_leaf_indices(expr, markup_direction=Down):
    r'''Label leaves in `expr` with leaf indices:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> labeltools.label_leaves_in_expr_with_leaf_indices(staff)
        >>> print format(staff)
        \new Staff {
            c'8 _ \markup { \small 0 }
            d'8 _ \markup { \small 1 }
            e'8 _ \markup { \small 2 }
            f'8 _ \markup { \small 3 }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    '''

    for i, leaf in enumerate(iterate(expr).by_class(scoretools.Leaf)):
        label = markuptools.MarkupCommand('small', str(i))
        markup = markuptools.Markup(label, markup_direction)
        attach(markup, leaf)



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.rhythmmakertools.IncisedRhythmMaker import IncisedRhythmMaker


class DivisionIncisedRhythmMaker(IncisedRhythmMaker):
    '''Abstract base class for rhythm-makers that incise every output 
    cell they produce.
    '''

    ### PRIVATE METHODS ###

    def _make_numeric_map(self, duration_pairs=None,
        prefix_talea=None, prefix_lengths=None,
        suffix_talea=None, suffix_lengths=None, prolation_addenda=None):
        numeric_map, prefix_talea_index, suffix_talea_index = [], 0, 0
        for pair_index, duration_pair in enumerate(duration_pairs):
            prefix_length, suffix_length = \
                prefix_lengths[pair_index], suffix_lengths[pair_index]
            prefix = prefix_talea[
                prefix_talea_index:prefix_talea_index+prefix_length]
            suffix = suffix_talea[
                suffix_talea_index:suffix_talea_index+suffix_length]
            prefix_talea_index += prefix_length
            suffix_talea_index += suffix_length
            prolation_addendum = prolation_addenda[pair_index]
            if isinstance(duration_pair, tuple):
                numerator = duration_pair[0] + (
                    prolation_addendum % duration_pair[0])
            else:
                numerator = duration_pair.numerator + (
                    prolation_addendum % duration_pair.numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, prefix, suffix)
            numeric_map.append(numeric_map_part)
        return numeric_map



# -*- encoding: utf-8 -*-
import fractions
from abjad.tools.abctools.AbjadObject import AbjadObject


class NonreducedFraction(AbjadObject, fractions.Fraction):
    r'''Initializes with an integer numerator and integer denominator:

    ::

        >>> mathtools.NonreducedFraction(3, 6)
        NonreducedFraction(3, 6)

    Initializes with only an integer denominator:

    ::

        >>> mathtools.NonreducedFraction(3)
        NonreducedFraction(3, 1)

    Initializes with an integer pair:

    ::

        >>> mathtools.NonreducedFraction((3, 6))
        NonreducedFraction(3, 6)

    Initializes with an integer singleton:

    ::

        >>> mathtools.NonreducedFraction((3,))
        NonreducedFraction(3, 1)

    Similar to built-in fraction except that numerator and denominator 
    do not reduce.

    Nonreduced fractions inherit from built-in fraction:

    ::

        >>> isinstance(mathtools.NonreducedFraction(3, 6), Fraction)
        True

    Nonreduced fractions are numbers:

    ::

        >>> import numbers

    ::

        >>> isinstance(mathtools.NonreducedFraction(3, 6), numbers.Number)
        True

    Nonreduced fraction initialization requires more function calls that 
    fraction initialization.
    But nonreduced fraction initialization is reasonably fast anyway:

    ::

        >>> import fractions
        >>> systemtools.IOManager.count_function_calls(
        ...     'fractions.Fraction(3, 6)', globals())
        13

    ::

        >>> systemtools.IOManager.count_function_calls(
        ...     'mathtools.NonreducedFraction(3, 6)', globals())
        30

    Nonreduced fractions are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_numerator', 
        '_denominator',
        )

    ### CONSTRUCTOR ###

    def __new__(cls, *args):
        from abjad.tools import mathtools
        from abjad.tools import sequencetools
        if len(args) == 1 and hasattr(args[0], 'numerator') and \
            hasattr(args[0], 'denominator'):
            numerator = args[0].numerator
            denominator = args[0].denominator
        elif len(args) == 1 and isinstance(args[0], int):
            numerator = args[0]
            denominator = 1
        elif len(args) == 1 and sequencetools.is_integer_singleton(args[0]):
            numerator = args[0][0]
            denominator = 1
        elif len(args) == 1 and sequencetools.is_integer_pair(args[0]):
            numerator, denominator = args[0]
        elif sequencetools.is_integer_pair(args):
            numerator = args[0]
            denominator = args[1]
        elif len(args) == 0:
            numerator = 0
            denominator = 1
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, args)
            raise ValueError(message)
        numerator *= mathtools.sign(denominator)
        denominator = abs(denominator)
        self = fractions.Fraction.__new__(cls, numerator, denominator)
        self._numerator = numerator
        self._denominator = denominator
        return self

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of nonreduced fraction.

        ::

            >>> abs(mathtools.NonreducedFraction(-3, 3))
            NonreducedFraction(3, 3)

        Returns nonreduced fraction.
        '''
        return type(self)((abs(self.numerator), self.denominator))

    def __add__(self, expr):
        r'''Adds `expr` to nonreduced fraction.

        ::

            >>> mathtools.NonreducedFraction(3, 3) + 1
            NonreducedFraction(6, 3)

        Adding two nonreduced fractions is fairly fast:

        ::

            >>> a = mathtools.NonreducedFraction(3, 6)
            >>> b = mathtools.NonreducedFraction(3, 12)
            >>> systemtools.IOManager.count_function_calls('a + b', globals())
            67

        Adding an integer is even faster:

        ::

            >>> systemtools.IOManager.count_function_calls('a + 10', globals())
            35

        Returns nonreduced fraction.
        '''
        from abjad.tools import mathtools
        if isinstance(expr, int):
            numerator = self.numerator + expr * self.denominator
            return NonreducedFraction(numerator, self.denominator)
        elif hasattr(expr, 'denominator'):
            if self.denominator == expr.denominator:
                numerator = self.numerator + expr.numerator
                return NonreducedFraction(numerator, self.denominator)
            else:
                denominator = mathtools.least_common_multiple(
                    self.denominator, expr.denominator)
                self_multiplier = denominator / self.denominator
                expr_multiplier = denominator / expr.denominator
                self_numerator = self_multiplier * self.numerator
                expr_numerator = expr_multiplier * expr.numerator
                return NonreducedFraction(
                    self_numerator + expr_numerator, denominator)
        else:
            raise ValueError(expr)

    def __div__(self, expr):
        r'''Divides nonreduced fraction by `expr`.

        ::

            >>> mathtools.NonreducedFraction(3, 3) / 1
            NonreducedFraction(3, 3)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator)
            expr = expr.reduce()
        fraction = self.reduce() / expr
        return self._fraction_with_denominator(fraction, max(denominators))

    def __eq__(self, expr):
        r'''True when `expr` equals nonreduced fraction.

        ::

            >>> mathtools.NonreducedFraction(3, 3) == 1
            True

        Returns boolean.
        '''
        return self.reduce() == expr

    def __format__(self, format_specification=''):
        r'''Formats nonreduced fraction.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)
            >>> print format(fraction)
            mathtools.NonreducedFraction(-6, 3)

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __ge__(self, expr):
        r'''True when nonreduced fraction is greater than or equal to `expr`.

        ::

            >>> mathtools.NonreducedFraction(3, 3) >= 1
            True

        Returns boolean.
        '''
        return self.reduce() >= expr

    def __gt__(self, expr):
        r'''True when nonreduced fraction is greater than `expr`.

        ::

            >>> mathtools.NonreducedFraction(3, 3) > 1
            False

        Returns boolean.
        '''
        return self.reduce() > expr

    def __le__(self, expr):
        r'''True when nonreduced fraction is less than or equal to `expr`.

        ::

            >>> mathtools.NonreducedFraction(3, 3) <= 1
            True

        Returns boolean.
        '''
        return self.reduce() <= expr

    def __lt__(self, expr):
        r'''True when nonreduced fraction is less than `expr`.

        ::

            >>> mathtools.NonreducedFraction(3, 3) < 1
            False

        Returns boolean.
        '''
        return self.reduce() < expr

    def __mul__(self, expr):
        r'''Multiplies nonreduced fraction by `expr`.

        ::

            >>> mathtools.NonreducedFraction(3, 3) * 3
            NonreducedFraction(9, 3)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator)
            expr = expr.reduce()
        fraction = self.reduce() * expr
        return self._fraction_with_denominator(fraction, max(denominators))

    def __ne__(self, expr):
        r'''True when `expr` does not equal nonreduced fraction.

        ::

            >>> mathtools.NonreducedFraction(3, 3) != 'foo'
            True

        Returns boolean.
        '''
        return not self == expr

    def __neg__(self):
        r'''Negates nonreduced fraction.

        ::

            >>> -mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(-3, 3)

        Returns nonreduced fraction.
        '''
        return type(self)((-self.numerator, self.denominator))

    def __pow__(self, expr):
        r'''Raises nonreduced fraction to `expr`.

        ::

            >>> mathtools.NonreducedFraction(3, 6) ** -1
            NonreducedFraction(6, 3)

        Returns nonreduced fraction.
        '''
        if expr == -1:
            return type(self)(self.denominator, self.numerator)
        return super(NonreducedFraction, self).__pow__(expr)

    def __radd__(self, expr):
        r'''Adds nonreduced fraction to `expr`.

        ::

            >>> 1 + mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(6, 3)

        Returns nonreduced fraction.
        '''
        return self + expr

    def __rdiv__(self, expr):
        r'''Divides `expr` by nonreduced fraction.

        ::

            >>> 1 / mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(3, 3)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator)
            expr = expr.reduce()
        fraction = expr / self.reduce()
        return self._fraction_with_denominator(fraction, max(denominators))

    def __repr__(self):
        r'''Gets interpreter representation of nonreduced fraction.

        ::

            >>> mathtools.NonreducedFraction(3, 6)
            NonreducedFraction(3, 6)

        Returns string.
        '''
        return '{}({}, {})'.format(
            type(self).__name__,
            self.numerator,
            self.denominator,
            )

    def __rmul__(self, expr):
        r'''Multiplies `expr` by nonreduced fraction.

        ::

            >>> 3 * mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(9, 3)

        Returns nonreduced fraction.
        '''
        return self * expr

    def __rsub__(self, expr):
        r'''Subtracts nonreduced fraction from `expr`.

        ::

            >>> 1 - mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(0, 3)

        Returns nonreduced fraction.
        '''
        return -self + expr

    def __str__(self):
        r'''String representation of nonreduced fraction.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> str(fraction)
            '-6/3'

        Returns string.
        '''
        return '{}/{}'.format(self.numerator, self.denominator)

    def __sub__(self, expr):
        r'''Subtracts `expr` from nonreduced fraction.

        ::

            >>> mathtools.NonreducedFraction(3, 3) - 2
            NonreducedFraction(-3, 3)

        Returns nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator)
            expr = expr.reduce()
        fraction = self.reduce() - expr
        return self._fraction_with_denominator(fraction, max(denominators))

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                self.numerator,
                self.denominator,
                ),
            )

    ### PRIVATE METHODS ###

    def _fraction_with_denominator(self, fraction, denominator):
        from abjad.tools import durationtools
        from abjad.tools import mathtools
        denominator = mathtools.least_common_multiple(
            denominator, fraction.denominator)
        return NonreducedFraction(fraction).with_denominator(denominator)

    ### PUBLIC PROPERTIES ###

    @property
    def denominator(self):
        r'''Denominator of nonreduced fraction.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.denominator
            3

        Returns positive integer.
        '''
        return self._denominator

    @property
    def imag(self):
        r'''Nonreduced fractions have no imaginary part.

        ::

            >>> fraction.imag
            0

        Returns zero.
        '''
        return fractions.Fraction.imag.fget(self)

    @property
    def numerator(self):
        r'''Numerator of nonreduced fraction.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.numerator
            -6

        Returns integer.
        '''
        return self._numerator

    @property
    def pair(self):
        r'''Read only pair of nonreduced fraction numerator and denominator.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.pair
            (-6, 3)

        Returns integer pair.
        '''
        return self.numerator, self.denominator

    @property
    def real(self):
        r'''Nonreduced fractions are their own real component.

        ::

            >>> fraction.real
            NonreducedFraction(-6, 3)

        Returns nonreduced fraction.
        '''
        return self

    ### PUBLIC METHODS ###

    def multiply_with_cross_cancelation(self, multiplier):
        '''Multiplies nonreduced fraction by `expr` with cross-cancelation.

        ::

            >>> fraction = mathtools.NonreducedFraction(4, 8)

        ::

            >>> fraction.multiply_with_cross_cancelation((2, 3))
            NonreducedFraction(4, 12)

        ::

            >>> fraction.multiply_with_cross_cancelation((4, 1))
            NonreducedFraction(4, 2)

        ::

            >>> fraction.multiply_with_cross_cancelation((3, 5))
            NonreducedFraction(12, 40)

        ::

            >>> fraction.multiply_with_cross_cancelation((6, 5))
            NonreducedFraction(12, 20)

        ::

            >>> fraction = mathtools.NonreducedFraction(5, 6)
            >>> fraction.multiply_with_cross_cancelation((6, 5))
            NonreducedFraction(1, 1)

        Returns nonreduced fraction.
        '''
        from abjad.tools import durationtools
        from abjad.tools import mathtools

        multiplier = durationtools.Multiplier(multiplier)

        self_numerator_factors = mathtools.factors(self.numerator)
        multiplier_denominator_factors = mathtools.factors(
            multiplier.denominator)
        for factor in multiplier_denominator_factors[:]:
            if factor in self_numerator_factors:
                self_numerator_factors.remove(factor)
                multiplier_denominator_factors.remove(factor)

        self_denominator_factors = mathtools.factors(self.denominator)
        multiplier_numerator_factors = mathtools.factors(multiplier.numerator)
        for factor in multiplier_numerator_factors[:]:
            if factor in self_denominator_factors:
                self_denominator_factors.remove(factor)
                multiplier_numerator_factors.remove(factor)

        result_numerator_factors = self_numerator_factors + \
            multiplier_numerator_factors
        result_denominator_factors = self_denominator_factors + \
            multiplier_denominator_factors

        result_numerator = 1
        for factor in result_numerator_factors:
            result_numerator *= factor

        result_denominator = 1
        for factor in result_denominator_factors:
            result_denominator *= factor

        return NonreducedFraction(result_numerator, result_denominator)

    def multiply_with_numerator_preservation(self, multiplier):
        r'''Multiplies nonreduced fraction by `multiplier` with numerator 
        preservation where possible.

        ::

            >>> fraction = mathtools.NonreducedFraction(9, 16)

        ::

            >>> fraction.multiply_with_numerator_preservation((2, 3))
            NonreducedFraction(9, 24)

        ::

            >>> fraction.multiply_with_numerator_preservation((1, 2))
            NonreducedFraction(9, 32)

        ::

            >>> fraction.multiply_with_numerator_preservation((5, 6))
            NonreducedFraction(45, 96)

        ::

            >>> fraction = mathtools.NonreducedFraction(3, 8)

        ::

            >>> fraction.multiply_with_numerator_preservation((2, 3))
            NonreducedFraction(3, 12)

        Returns nonreduced fraction.
        '''
        from abjad.tools import durationtools

        multiplier = durationtools.Multiplier(multiplier)
        self_denominator = self.denominator
        candidate_result_denominator = self_denominator / multiplier

        if candidate_result_denominator.denominator == 1:
            return NonreducedFraction(
                self.numerator, candidate_result_denominator.numerator)
        else:
            result_numerator = \
                self.numerator * candidate_result_denominator.denominator
            result_denominator = candidate_result_denominator.numerator
            return NonreducedFraction(result_numerator, result_denominator)

    def multiply_without_reducing(self, expr):
        r'''Multiplies nonreduced fraction by `expr` without reducing.

        ::

            >>> fraction = mathtools.NonreducedFraction(3, 8)

        ::

            >>> fraction.multiply_without_reducing((3, 3))
            NonreducedFraction(9, 24)

        ::

            >>> fraction = mathtools.NonreducedFraction(4, 8)

        ::

            >>> fraction.multiply_without_reducing((4, 5))
            NonreducedFraction(16, 40)

        ::

            >>> fraction.multiply_without_reducing((3, 4))
            NonreducedFraction(12, 32)

        Returns nonreduced fraction.
        '''
        expr = NonreducedFraction(expr)
        numerator = self.numerator * expr.numerator
        denominator = self.denominator * expr.denominator
        return NonreducedFraction(numerator, denominator)

    def reduce(self):
        r'''Reduces nonreduced fraction.

        ::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.reduce()
            Fraction(-2, 1)

        Returns fraction.
        '''
        return fractions.Fraction(self.numerator, self.denominator)

    def with_denominator(self, denominator):
        r'''Returns new nonreduced fraction with integer `denominator`.

        ::

            >>> mathtools.NonreducedFraction(3, 6).with_denominator(12)
            NonreducedFraction(6, 12)

        Returns nonreduced fraction.
        '''
        from abjad.tools import durationtools
        n, d = self.pair
        multiplier = durationtools.Multiplier(denominator, d)
        new_numerator = multiplier * n
        new_denominator = multiplier * d
        if new_numerator.denominator == 1 and \
            new_denominator.denominator == 1:
            return type(self)(
                new_numerator.numerator, new_denominator.numerator)
        else:
            return type(self)(n, d)

    def with_multiple_of_denominator(self, denominator):
        r'''Returns new nonreduced fraction with multiple of integer 
        `denominator`.

        ::

            >>> fraction = mathtools.NonreducedFraction(3, 6)

        ::

            >>> fraction.with_multiple_of_denominator(5)
            NonreducedFraction(5, 10)

        Returns nonreduced fraction.
        '''
        result = self.with_denominator(denominator)
        while not result.denominator == denominator:
            denominator *= 2
            result = result.with_denominator(denominator)
        return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_color_contents_of_container_01():

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))
    labeltools.color_contents_of_container(staff[1], 'blue')

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \override Accidental #'color = #blue
                \override Beam #'color = #blue
                \override Dots #'color = #blue
                \override NoteHead #'color = #blue
                \override Rest #'color = #blue
                \override Stem #'color = #blue
                \override TupletBracket #'color = #blue
                \override TupletNumber #'color = #blue
                e'8
                f'8
                \revert Accidental #'color
                \revert Beam #'color
                \revert Dots #'color
                \revert NoteHead #'color
                \revert Rest #'color
                \revert Stem #'color
                \revert TupletBracket #'color
                \revert TupletNumber #'color
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-
import copy
from abjad import *


def test_pitchtools_NamedPitch___deepcopy___01():

    pitch = NamedPitch(13)
    new = copy.deepcopy(pitch)

    assert new is not pitch
    assert new.accidental is not pitch.accidental



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class IsAtSoundingPitch(AbjadObject):
    r'''Is at sounding pitch indicator.

    ::

        >>> indicator = indicatortools.IsAtSoundingPitch()

    Attach to score selection to denote music written at sounding pitch.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_PayloadTree_to_nested_lists_01():

    sequence_1 = [[0, 1], [2, 3]]
    tree_1 = datastructuretools.PayloadTree(sequence_1)

    sequence_2 = tree_1.to_nested_lists()
    tree_2 = datastructuretools.PayloadTree(sequence_2)

    assert isinstance(sequence_2, list)
    assert sequence_1 == sequence_2
    assert tree_1 == tree_2



# -*- encoding: utf-8 -*-
'''Page layout tools.

   This package depends on the following:

      * rational numbers
'''

from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
	__path__[0],
	globals(),
	)

_documentation_section = 'core'



# -*- encoding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import mathtools
from abjad.tools.durationtools.Duration import Duration


class Multiplier(Duration):
    '''A multiplier.

    ::

        >>> Multiplier(2, 3)
        Multiplier(2, 3)

    '''

    ### SPECIAL METHODS ###

    def __mul__(self, *args):
        r'''Multiplier times duration gives duration.

        Returns duration.
        '''
        if len(args) == 1 and type(args[0]) is Duration:
            return Duration(Duration.__mul__(self, *args))
        else:
            return Duration.__mul__(self, *args)

    ### PUBLIC PROPERTIES ###

    @property
    def is_proper_tuplet_multiplier(self):
        '''True when mutliplier is greater than ``1/2`` and less than ``2``.
        Otherwise false:

        ::

            >>> Multiplier(3, 2).is_proper_tuplet_multiplier
            True

        Returns boolean.
        '''
        return type(self)(1, 2) < self < type(self)(2)



# -*- encoding: utf-8 -*-
import math


def partition_sequence_extended_to_counts(sequence, counts, overhang=True):
    '''Partition sequence extended to counts.

    ..  container:: example

        **Example 1.** Partition sequence extended to counts with overhang:

        ::

            >>> sequencetools.partition_sequence_extended_to_counts(
            ...     (1, 2, 3, 4), 
            ...     (6, 6, 6), 
            ...     overhang=True,
            ...     )
            [(1, 2, 3, 4, 1, 2), (3, 4, 1, 2, 3, 4), (1, 2, 3, 4, 1, 2), (3, 4)]

    ..  container:: example

        **Example 2.** Partition sequence extended to coutns without overhang:

        ::

            >>> sequencetools.partition_sequence_extended_to_counts(
            ...     (1, 2, 3, 4), 
            ...     (6, 6, 6), 
            ...     overhang=False,
            ...     )
            [(1, 2, 3, 4, 1, 2), (3, 4, 1, 2, 3, 4), (1, 2, 3, 4, 1, 2)]

    Returns sequence of sequence objects.
    '''
    from abjad.tools import sequencetools

    n = int(math.ceil(float(sum(counts)) / len(sequence)))

    sequence = sequencetools.repeat_sequence_n_times(sequence, n)

    return sequencetools.partition_sequence_by_counts(
        sequence, 
        counts, 
        cyclic=False, 
        overhang=overhang,
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_partition_sequence_extened_to_counts_01():

    parts = sequencetools.partition_sequence_extended_to_counts([1, 2, 3, 4], [6, 6, 6], overhang=True)

    assert parts == [[1, 2, 3, 4, 1, 2], [3, 4, 1, 2, 3, 4], [1, 2, 3, 4, 1, 2], [3, 4]]


def test_sequencetools_partition_sequence_extened_to_counts_02():

    parts = sequencetools.partition_sequence_extended_to_counts([1, 2, 3, 4], [6, 6, 6], overhang=False)

    assert parts == [[1, 2, 3, 4, 1, 2], [3, 4, 1, 2, 3, 4], [1, 2, 3, 4, 1, 2]]



# -*- encoding: utf-8 -*-


def zip_sequences_without_truncation(*sequences):
    '''Zip `sequences` nontruncating:

    ::

        >>> sequencetools.zip_sequences_without_truncation(
        ...     [1, 2, 3, 4], [11, 12, 13], [21, 22, 23])
        [(1, 11, 21), (2, 12, 22), (3, 13, 23), (4,)]

    Lengths of the tuples returned may differ but will always be
    greater than or equal to ``1``.

    Returns list of tuples.
    '''

    result = []

    max_length = max([len(l) for l in sequences])
    for i in range(max_length):
        part = []
        for l in sequences:
            try:
                part.append(l[i])
            except IndexError:
                pass
        result.append(tuple(part))

    return result



# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools


def list_ordered_named_pitch_pairs_from_expr_1_to_expr_2(expr_1, expr_2):
    '''List ordered named pitch pairs from `expr_1` to `expr_2`:

    ::

        >>> chord_1 = Chord([0, 1, 2], (1, 4))
        >>> chord_2 = Chord([3, 4], (1, 4))

    ::

        >>> for pair in pitchtools.list_ordered_named_pitch_pairs_from_expr_1_to_expr_2(
        ...     chord_1, chord_2):
        ...     pair
        (NamedPitch("c'"), NamedPitch("ef'"))
        (NamedPitch("c'"), NamedPitch("e'"))
        (NamedPitch("cs'"), NamedPitch("ef'"))
        (NamedPitch("cs'"), NamedPitch("e'"))
        (NamedPitch("d'"), NamedPitch("ef'"))
        (NamedPitch("d'"), NamedPitch("e'"))

    Returns generator.
    '''
    from abjad.tools import pitchtools

    pitches_1 = sorted(pitchtools.list_named_pitches_in_expr(expr_1))
    pitches_2 = sorted(pitchtools.list_named_pitches_in_expr(expr_2))
    for pair in sequencetools.yield_all_pairs_between_sequences(pitches_1, pitches_2):
        yield pair



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitcharraytools_PitchArrayCell_is_last_in_row_01():

    array = pitcharraytools.PitchArray([[1, 2, 1], [2, 1, 1]])

    '''
    [] [      ] []
    [      ] [] []
    '''

    assert not array[0].cells[0].is_last_in_row
    assert not array[0].cells[1].is_last_in_row
    assert array[0].cells[2].is_last_in_row

    assert not array[1].cells[0].is_last_in_row
    assert not array[1].cells[1].is_last_in_row
    assert array[1].cells[2].is_last_in_row



def test_pitcharraytools_PitchArrayCell_is_last_in_row_02():

    cell = pitcharraytools.PitchArrayCell([NamedPitch(1)])

    assert not cell.is_last_in_row



# -*- encoding: utf-8 -*-
from abjad import *


# What's the best way to test play?

def test_topleveltools_play_01():
    r'''A note can be played.
    '''
    note = Note(1, (1, 2))
    play(note)


def test_topleveltools_play_02():
    r'''A score can be played.
    '''
    notes = [Note(i, (1, 64)) for i in range(10)]
    score = Score([Staff(notes)])
    play(score)



# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import select


def get_one_indexed_measure_number_in_expr(expr, measure_number):
    r'''Gets one-indexed `measure_number` in `expr`.

    ::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }

    ::

        >>> scoretools.get_one_indexed_measure_number_in_expr(staff, 3)
        Measure((2, 8), "g'8 a'8")

    Note that measures number from ``1``.
    '''
    from abjad.tools import scoretools

    # check input
    if measure_number < 1:
        message = 'measure numbers allow only positive integers.'
        raise ValueError(message)

    # calculate measure index
    measure_index = measure_number - 1

    # return measure
    selection = select(expr)
    return selection._get_component(scoretools.Measure, measure_index)



# -*- encoding: utf-8 -*-
import abc
import collections
import types
from abjad.tools.datastructuretools import TypedTuple


class Segment(TypedTuple):
    r'''Music-theoretic segment base class.
    '''

    ### CLASS VARIABLES ##

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self, 
        tokens=None, 
        item_class=None, 
        custom_identifier=None,
        ):
        from abjad.tools import datastructuretools
        from abjad.tools import pitchtools 
        if isinstance(tokens, str):
            tokens = tokens.split()
        elif isinstance(tokens, (
            collections.Iterator,
            types.GeneratorType,
            )):
            tokens = [token for token in tokens]
        if item_class is None:
            item_class = self._named_item_class
            if tokens is not None:
                if isinstance(tokens, datastructuretools.TypedCollection) and \
                    issubclass(tokens.item_class, self._parent_item_class):
                    item_class = tokens.item_class
                elif len(tokens):
                    if isinstance(tokens, collections.Set):
                        tokens = tuple(tokens)
                    if isinstance(tokens[0], str):
                        item_class = self._named_item_class
                    elif isinstance(tokens[0], (int, float)):
                        item_class = self._numbered_item_class
                    elif isinstance(tokens[0], self._parent_item_class):
                        item_class = type(tokens[0])
        assert issubclass(item_class, self._parent_item_class)
        TypedTuple.__init__(
            self,
            tokens=tokens,
            item_class=item_class,
            custom_identifier=custom_identifier,
            )

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''String representation of segment.

        Returns string.
        '''
        parts = [str(x) for x in self]
        return '<{}>'.format(', '.join(parts))
        
    ### PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _named_item_class(self):
        raise NotImplementedError
    
    @abc.abstractproperty
    def _numbered_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _parent_item_class(self):
        raise NotImplementedError

    @property
    def _repr_specification(self):
        tokens = []
        if self.item_class.__name__.startswith('Named'):
            tokens = [str(x) for x in self]
        else:
            tokens = [abs(x) for x in self]
        return self._storage_format_specification.__makenew__(
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                tokens,
                ),
            )

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def from_selection(
        cls, 
        selection, 
        item_class=None, 
        custom_identifier=None,
        ):
        r'''Makes segment from `selection`.

        Returns new segment.
        '''
        raise NotImplementedError

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def has_duplicates(self):
        r'''True when segment has duplicates. Otherwise false.

        Returns boolean.
        '''
        raise NotImplementedError 



# -*- encoding: utf-8 -*-


def negate_absolute_value_of_sequence_elements_cyclically(sequence, indices, period):
    '''Negate the absolute value of `sequence` elements at `indices` cyclically
    according to `period`:

    ::

        >>> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]

    ::

        >>> sequencetools.negate_absolute_value_of_sequence_elements_cyclically(
        ...     sequence, [0, 1, 2], 5)
        [-1, -2, -3, 4, 5, -6, -7, -8, -9, -10]

    Returns newly constructed list.
    '''

    result = []

    for i, element in enumerate(sequence):
        if (i in indices) or (period and i % period in indices):
            result.append(-abs(element))
        else:
            result.append(element)

    return result



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_sequencetools_yield_all_rotations_of_sequence_01():
    r'''Yield all rotations of list.
    '''

    rotations = list(sequencetools.yield_all_rotations_of_sequence([1, 2, 3, 4], -1))
    assert rotations == [[1, 2, 3, 4], [2, 3, 4, 1], [3, 4, 1, 2], [4, 1, 2, 3]]


def test_sequencetools_yield_all_rotations_of_sequence_02():
    r'''Yield all rotations of tuple.
    '''

    rotations = list(sequencetools.yield_all_rotations_of_sequence((1, 2, 3, 4), -1))
    assert rotations == [(1, 2, 3, 4), (2, 3, 4, 1), (3, 4, 1, 2), (4, 1, 2, 3)]



# -*- encoding: utf-8 -*-
import numbers


def all_are_numbers(expr):
    '''True when `expr` is a sequence and all elements in `expr` are numbers:

    ::

        >>> sequencetools.all_are_numbers([1, 2, 3.0, Fraction(13, 8)])
        True

    True when `expr` is an empty sequence:

    ::

        >>> sequencetools.all_are_numbers([])
        True

    Otherwise false:

    ::

        >>> sequencetools.all_are_numbers(17)
        False

    Returns boolean.
    '''

    try:
        return all(isinstance(x, numbers.Number) for x in expr)
    except TypeError:
        return False



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__indicators__Clef_01():

    target = Staff([Note(0, 1)])
    clef = Clef('bass')
    attach(clef, target[0])

    assert systemtools.TestManager.compare(
        target,
        r'''
        \new Staff {
            \clef "bass"
            c'1
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result
    clefs = inspect(result[0]).get_indicators(Clef)
    assert len(clefs) == 1



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.demos.desordre.make_desordre_staff import make_desordre_staff


def make_desordre_score(pitches):
    '''Returns a complete piano staff with Ligeti music.
    '''

    assert len(pitches) == 2
    piano_staff = scoretools.PianoStaff()

    # build the music
    for hand in pitches:
        staff = make_desordre_staff(hand)
        piano_staff.append(staff)

    # set clef and key signature to left hand staff
    clef = indicatortools.Clef('bass')
    attach(clef, piano_staff[1])
    key_signature = KeySignature('b', 'major')
    attach(key_signature, piano_staff[1])

    # wrap the piano staff in a score
    score = scoretools.Score([piano_staff])

    return score



# -*- encoding: utf-8 -*-
from abjad import *


def test_stringtools_is_snake_case_package_name_01():

    assert stringtools.is_snake_case_package_name('foo.bar.blah')
    assert stringtools.is_snake_case_package_name('foo.bar_blah')
    assert stringtools.is_snake_case_package_name('foo_bar.blah')
    assert stringtools.is_snake_case_package_name('foo')


def test_stringtools_is_snake_case_package_name_02():

    assert stringtools.is_snake_case_package_name('')


def test_stringtools_is_snake_case_package_name_03():

    assert not stringtools.is_snake_case_package_name('foo.bar.Blah')
    assert not stringtools.is_snake_case_package_name('foo.bar.BlahPackage')
    assert not stringtools.is_snake_case_package_name('Foo')
    assert not stringtools.is_snake_case_package_name('Foo.Bar')



# -*- encoding: utf-8 -*-
from abjad.tools import abctools


class CyclicList(abctools.AbjadObject, list):
    '''A cyclic list.

    ::

        >>> cyclic_list = datastructuretools.CyclicList('abcd')

    ::

        >>> cyclic_list
        CyclicList(['a', 'b', 'c', 'd'])

    ::

        >>> for x in range(8):
        ...     print x, cyclic_list[x]
        ...
        0 a
        1 b
        2 c
        3 d
        4 a
        5 b
        6 c
        7 d

    Cyclic lists overload the item-getting method of built-in lists.

    Cyclic lists return a value for any integer index.

    Cyclic lists otherwise behave exactly like built-in lists.
    '''

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` has items equal to those of this cyclic list.
        Otherwise false.

        Returns boolean.
        '''
        return list.__eq__(self, expr)
    
    def __getitem__(self, i):
        r'''Gets `i` from cyclic list.

        Raise index error when `i` can not be found in cyclic list.

        Returns item.
        '''
        if len(self):
            i = i % len(self)
            return list.__getitem__(self, i)
        else:
            raise IndexError

    def __getslice__(self, start_index, stop_index):
        r'''Gets items in cyclic list from `start_index` to `stop_index`.

        Returns list.
        '''
        result = []
        result = [self[n] for n in range(start_index, stop_index)]
        return result

    def __str__(self):
        r'''String representation of cyclic list.

        Returns string.
        '''
        return '[{!s}]' % ', '.join([str(x) for x in self])

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                list(self),
                ),
            )



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools


def test_sequencetools_all_are_nonnegative_integer_powers_of_two_01():
    r'''True when all elements in sequence are nonnegative integer powers of two.
    '''

    assert sequencetools.all_are_nonnegative_integer_powers_of_two([1, 2, 256, 8, 16, 16, 16])


def test_sequencetools_all_are_nonnegative_integer_powers_of_two_02():
    r'''True on empty sequence.
    '''

    assert sequencetools.all_are_nonnegative_integer_powers_of_two([])


def test_sequencetools_all_are_nonnegative_integer_powers_of_two_03():
    r'''Otherwise false.
    '''

    assert not sequencetools.all_are_nonnegative_integer_powers_of_two([3])
    assert not sequencetools.all_are_nonnegative_integer_powers_of_two([1, 2, 4, 8, 16, 17])


def test_sequencetools_all_are_nonnegative_integer_powers_of_two_04():
    r'''False when expr is not a sequence.
    '''

    assert not sequencetools.all_are_nonnegative_integer_powers_of_two(16)



# -*- encoding: utf-8 -*-
from abjad.tools.pitchtools.Set import Set


class PitchClassSet(Set):
    '''A pitch-class set.

    ::

        >>> numbered_pitch_class_set = pitchtools.PitchClassSet(
        ...     tokens=[-2, -1.5, 6, 7, -1.5, 7],
        ...     item_class=pitchtools.NumberedPitchClass,
        ...     )
        >>> numbered_pitch_class_set
        PitchClassSet([6, 7, 10, 10.5])

    ::

        >>> named_pitch_class_set = pitchtools.PitchClassSet(
        ...     tokens=['c', 'ef', 'bqs,', 'd'],
        ...     item_class=pitchtools.NamedPitchClass,
        ...     )
        >>> named_pitch_class_set
        PitchClassSet(['c', 'd', 'ef', 'bqs'])

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __hash__(self):
        r'''Hashes pitch-class set.

        Returns integer.
        '''
        return hash(repr(self))

    ### PRIVATE METHODS ###

    def _sort_self(self):
        from abjad.tools import pitchtools
        def helper(x, y):
            return cmp(
                pitchtools.NamedPitch(pitchtools.NamedPitchClass(x), 0),
                pitchtools.NamedPitch(pitchtools.NamedPitchClass(y), 0)
                )
        result = list(self)
        result.sort(helper)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass
    
    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.PitchClass

    @property
    def _repr_specification(self):
        tokens = []
        if self.item_class.__name__.startswith('Named'):
            tokens = [str(x) for x in sorted(
                self, key=lambda x: x.pitch_class_number)]
        else:
            tokens = sorted([abs(x) for x in self])
        return self._storage_format_specification.__makenew__(
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                tokens,
                ),
            )

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        cls, 
        selection, 
        item_class=None, 
        custom_identifier=None,
        ):
        r'''Makes pitch-class set from `selection`.

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> pitchtools.PitchClassSet.from_selection(selection)
            PitchClassSet(['c', 'd', 'fs', 'g', 'a', 'b'])
        
        Returns pitch-class set.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return cls(
            tokens=pitch_segment,
            item_class=item_class,
            custom_identifier=custom_identifier,
            )

    def invert(self):
        r'''Inverts pitch-class set.

        ::

            >>> pitchtools.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     ).invert()
            PitchClassSet([1.5, 2, 5, 6])

        Returns numbered pitch-class set.
        '''
        return type(self)([pc.invert() for pc in self])

    def is_transposed_subset(self, pcset):
        r'''True when pitch-class set is transposed subset of `pcset`.
        Otherwise false:

        ::

            >>> pitch_class_set_1 = pitchtools.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_set_2 = pitchtools.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7, 7.5, 8],
            ...     )

        ::

            >>> pitch_class_set_1.is_transposed_subset(pitch_class_set_2)
            True

        Returns boolean.
        '''
        for n in range(12):
            if self.transpose(n).issubset(pcset):
                return True
        return False

    def is_transposed_superset(self, pcset):
        r'''True when pitch-class set is transposed superset of `pcset`.
        Otherwise false:

        ::

            >>> pitch_class_set_1 = pitchtools.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_set_2 = pitchtools.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7, 7.5, 8],
            ...     )

        ::

            >>> pitch_class_set_2.is_transposed_superset(pitch_class_set_1)
            True

        Returns boolean.
        '''
        for n in range(12):
            if self.transpose(n).issuperset(pcset):
                return True
        return False

    def multiply(self, n):
        r'''Multiplies pitch-class set by `n`.

        ::

            >>> pitchtools.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     ).multiply(5)
            PitchClassSet([2, 4.5, 6, 11])

        Returns new pitch-class set.
        '''
        tokens = (pitch_class.multiply(n) for pitch_class in self)
        return self.__makenew__(tokens=tokens)

    def order_by(self, pitch_class_segment):
        r'''Orders pitch-class set by `pitch_class_segment`.

        Returns pitch-class segment.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import sequencetools
        if not len(self) == len(pitch_class_segment):
            messaege = 'set and segment must be on equal length.'
            raise ValueError(message)
        for pitch_classes in sequencetools.yield_all_permutations_of_sequence(
            tuple(self)):
            candidate_pitch_class_segment = \
                pitchtools.PitchClassSegment(pitch_classes)
            if candidate_pitch_class_segment.is_equivalent_under_transposition(
                pitch_class_segment):
                return candidate_pitch_class_segment
        message = 'named pitch-class set {} can not order by '
        message += 'named pitch-class segment {}.'
        message = message.format(self, pitch_class_semgent)
        raise ValueError(message)

    def transpose(self, expr):
        r'''Transposes all pitch-classes in pitch-class set by `expr`.

        Returns new pitch-class set.
        '''
        tokens = (pitch_class + expr for pitch_class in self)
        return self.__makenew__(tokens=tokens)



# -*- encoding: utf-8 -*-
import collections
from abjad.tools.datastructuretools.TypedCollection import TypedCollection


class TypedTuple(TypedCollection):
    r'''A typed tuple.
    '''

    ### CLASS VARIABLES ### 

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, tokens=None, item_class=None, custom_identifier=None):
        TypedCollection.__init__(self, 
            item_class=item_class, 
            custom_identifier=custom_identifier,
            tokens=tokens,
            )
        tokens = tokens or []
        self._collection = tuple(self._item_callable(token) 
            for token in tokens)

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Adds typed tuple to `expr`.

        Returns new typed tuple.
        '''
        if isinstance(expr, type(self)):
            tokens = expr._collection
            return self.__makenew__(tokens=self._collection[:] + tokens)
        elif isinstance(expr, type(self._collection)):
            tokens = expr[:]
            return self.__makenew__(tokens=self._collection[:] + tokens)
        raise NotImplementedError

    def __contains__(self, token):
        r'''Change `token` to item and return true if item exists in
        collection.

        Returns none.
        '''
        try:
            item = self._item_callable(token)
        except ValueError:
            return False
        return self._collection.__contains__(item)

    def __getitem__(self, i):
        '''Gets `i` from type tuple.

        Returns item.
        '''
        return self._collection[i]

    def __getslice__(self, start, stop):
        r'''Gets slice from `start` to `stop` in typed tuple.

        Returns new typed tuple.
        '''
        return self.__makenew__(tokens=self._collection[start:stop])

    def __hash__(self):
        r'''Hashes typed tuple.

        Returns integer.
        '''
        return hash((
            type(self), 
            self._collection,
            self.item_class,
            self.name,
            ))

    def __mul__(self, expr):
        r'''Multiplies typed tuple by `expr`.

        Returns new typed tuple.
        '''
        tokens = self._collection * expr
        return self.__makenew__(tokens=tokens)

    def __rmul__(self, expr):
        r'''Multiplies `expr` by typed tuple.

        Returns new typed tuple.
        '''
        return self.__mul__(expr)

    ### PUBLIC METHODS ###

    def count(self, token):
        r'''Changes `token` to item.
        
        Returns count in collection.
        '''
        item = self._item_callable(token)
        return self._collection.count(item)

    def index(self, token):
        r'''Changes `token` to item.
        
        Returns index in collection.
        '''
        item = self._item_callable(token)
        return self._collection.index(item)


collections.Sequence.register(TypedTuple)



# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import override


def color_contents_of_container(container, color):
    r'''Color contents of `container`:

    ::

        >>> measure = Measure((2, 8), "c'8 d'8")

    ::

        >>> labeltools.color_contents_of_container(measure, 'red')
        Measure((2, 8), "c'8 d'8")

    ..  doctest::

        >>> print format(measure)
        {
            \override Accidental #'color = #red
            \override Beam #'color = #red
            \override Dots #'color = #red
            \override NoteHead #'color = #red
            \override Rest #'color = #red
            \override Stem #'color = #red
            \override TupletBracket #'color = #red
            \override TupletNumber #'color = #red
            \time 2/8
            c'8
            d'8
            \revert Accidental #'color
            \revert Beam #'color
            \revert Dots #'color
            \revert NoteHead #'color
            \revert Rest #'color
            \revert Stem #'color
            \revert TupletBracket #'color
            \revert TupletNumber #'color
        }

    ::

        >>> show(measure) # doctest: +SKIP

    Returns none.
    '''

    override(container).accidental.color = color
    override(container).beam.color = color
    override(container).dots.color = color
    override(container).note_head.color = color
    override(container).rest.color = color
    override(container).stem.color = color
    override(container).tuplet_bracket.color = color
    override(container).tuplet_number.color = color

    return container



# -*- encoding: utf-8 -*-
import collections
from abjad.tools import abctools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.topleveltools import iterate


class IterationAgent(abctools.AbjadObject):
    r'''A wrapper around the Abjad iteration methods.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> iterate(staff[2:])
            IterationAgent(client=SliceSelection(Note("d'4"), Note("f'4")))

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(self, client=None):
        self._client = client

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        r'''Client of iteration agent.

        Returns selection.
        '''
        return self._client

    ### PUBLIC METHODS ###

    def by_class(
        self,
        prototype=None,
        reverse=False,
        start=0,
        stop=None,
        ):
        r'''Iterate components forward in `expr`.

        ::

            >>> staff = Staff()
            >>> staff.append(Measure((2, 8), "c'8 d'8"))
            >>> staff.append(Measure((2, 8), "e'8 f'8"))
            >>> staff.append(Measure((2, 8), "g'8 a'8"))

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 2/8
                    c'8
                    d'8
                }
                {
                    e'8
                    f'8
                }
                {
                    g'8
                    a'8
                }
            }

        ::

            >>> for note in iterate(staff).by_class(Note):
            ...     note
            ...
            Note("c'8")
            Note("d'8")
            Note("e'8")
            Note("f'8")
            Note("g'8")
            Note("a'8")

        Use optional `start` and `stop` keyword parameters to control
        start and stop indices of iteration:

        ::

            >>> for note in iterate(staff).by_class(
            ...     Note, start=0, stop=3):
            ...     note
            ...
            Note("c'8")
            Note("d'8")
            Note("e'8")

        ::

            >>> for note in iterate(staff).by_class(
            ...     Note, start=2, stop=4):
            ...     note
            ...
            Note("e'8")
            Note("f'8")

        Yield right-to-left notes in `expr`:

        ::

            >>> staff = Staff()
            >>> staff.append(Measure((2, 8), "c'8 d'8"))
            >>> staff.append(Measure((2, 8), "e'8 f'8"))
            >>> staff.append(Measure((2, 8), "g'8 a'8"))

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 2/8
                    c'8
                    d'8
                }
                {
                    e'8
                    f'8
                }
                {
                    g'8
                    a'8
                }
            }

        ::

            >>> for note in iterate(staff).by_class(
            ...     Note, reverse=True):
            ...     note
            ...
            Note("a'8")
            Note("g'8")
            Note("f'8")
            Note("e'8")
            Note("d'8")
            Note("c'8")

        Use optional `start` and `stop` keyword parameters to control
        indices of iteration:

        ::

            >>> for note in iterate(staff).by_class(
            ...     Note, reverse=True, start=3):
            ...     note
            ...
            Note("e'8")
            Note("d'8")
            Note("c'8")

        ::

            >>> for note in iterate(staff).by_class(
            ...     Note, reverse=True, start=0, stop=3):
            ...     note
            ...
            Note("a'8")
            Note("g'8")
            Note("f'8")

        ::

            >>> for note in iterate(staff).by_class(
            ...     Note, reverse=True, start=2, stop=4):
            ...     note
            ...
            Note("f'8")
            Note("e'8")

        Iterates across different logical voices.

        Returns generator.
        '''
        prototype = prototype or scoretools.Component
        def component_iterator(expr, component_class, reverse=False):
            if isinstance(expr, component_class):
                yield expr
            if isinstance(expr, (list, tuple, spannertools.Spanner)) or \
                hasattr(expr, '_music'):
                if hasattr(expr, '_music'):
                    expr = expr._music
                if reverse:
                    expr = reversed(expr)
                for component in expr:
                    for x in component_iterator(
                        component, component_class, reverse=reverse):
                        yield x

        def subrange(iter, start=0, stop=None):
            # if start<0, then 'stop-start' gives a funny result
            # don not have to check stop>=start
            # because xrange(stop-start) already handles that
            assert 0 <= start

            try:
                # skip the first few elements, up to 'start' of them:
                for i in xrange(start):
                    # no yield to swallow the results
                    iter.next()

                # now generate (stop-start) elements
                # (or all elements if stop is none)
                if stop is None:
                    for x in iter:
                        yield x
                else:
                    for i in xrange(stop - start):
                        yield iter.next()
            except StopIteration:
                # this happens if we exhaust the list before
                # we generate a total of 'stop' elements
                pass

        return subrange(
            component_iterator(
                self._client,
                prototype,
                reverse=reverse),
            start,
            stop,
            )

    def by_components_and_grace_containers(self, prototype=None):
        r'''Iterate components of `component_class` forward in `expr`:

        ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> beam = spannertools.Beam()
            >>> attach(beam, voice[:])

        ::

            >>> grace_notes = [Note("c'16"), Note("d'16")]
            >>> grace = scoretools.GraceContainer(
            ...     grace_notes,
            ...     kind='grace',
            ...     )
            >>> attach(grace, voice[1])

        ::

            >>> after_grace_notes = [Note("e'16"), Note("f'16")]
            >>> after_grace = scoretools.GraceContainer(
            ...     after_grace_notes,
            ...     kind='after')
            >>> attach(after_grace, voice[1])

        ..  doctest::

            >>> print format(voice)
            \new Voice {
                c'8 [
                \grace {
                    c'16
                    d'16
                }
                \afterGrace
                d'8
                {
                    e'16
                    f'16
                }
                e'8
                f'8 ]
            }

        ::

            >>> x = iterate(voice).by_components_and_grace_containers(Note)
            >>> for note in x:
            ...     note
            ...
            Note("c'8")
            Note("c'16")
            Note("d'16")
            Note("d'8")
            Note("e'16")
            Note("f'16")
            Note("e'8")
            Note("f'8")

        Include grace leaves before main leaves.

        Include grace leaves after main leaves.
        '''
        prototype = prototype or scoretools.Leaf
        if self._client._grace is not None:
            for component in self._client._grace:
                for x in iterate(component).by_components_and_grace_containers(
                    prototype,
                    ):
                    yield x
            if isinstance(self._client, prototype):
                yield self._client
        if self._client._after_grace is not None:
            for component in self._client._after_grace:
                for x in iterate(component).by_components_and_grace_containers(
                    prototype,
                    ):
                    yield x
        elif isinstance(self._client, prototype):
            yield self._client
        if isinstance(self._client, (list, tuple)):
            for component in self._client:
                for x in iterate(component).by_components_and_grace_containers(
                    prototype,
                    ):
                    yield x
        if hasattr(self._client, '_music'):
            for component in self._client._music:
                for x in iterate(component).by_components_and_grace_containers(
                    prototype,
                    ):
                    yield x

    def by_leaf_pair(self):
        r'''Iterate leaf pairs forward in `expr`:

        ::

            >>> score = Score([])
            >>> notes = [Note("c'8"), Note("d'8"), Note("e'8"),
            ...     Note("f'8"), Note("g'4")]
            >>> score.append(Staff(notes))
            >>> notes = [Note(x, (1, 4)) for x in [-12, -15, -17]]
            >>> score.append(Staff(notes))
            >>> clef = Clef('bass')
            >>> attach(clef, score[1])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print format(score)
            \new Score <<
                \new Staff {
                    c'8
                    d'8
                    e'8
                    f'8
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    c4
                    a,4
                    g,4
                }
            >>

        ::

            >>> for pair in iterate(score).by_leaf_pair():
            ...        pair
            (Note("c'8"), Note('c4'))
            (Note("c'8"), Note("d'8"))
            (Note('c4'), Note("d'8"))
            (Note("d'8"), Note("e'8"))
            (Note("d'8"), Note('a,4'))
            (Note('c4'), Note("e'8"))
            (Note('c4'), Note('a,4'))
            (Note("e'8"), Note('a,4'))
            (Note("e'8"), Note("f'8"))
            (Note('a,4'), Note("f'8"))
            (Note("f'8"), Note("g'4"))
            (Note("f'8"), Note('g,4'))
            (Note('a,4'), Note("g'4"))
            (Note('a,4'), Note('g,4'))
            (Note("g'4"), Note('g,4'))

        Iterate leaf pairs left-to-right and top-to-bottom.

        Returns generator.
        '''
        vertical_moments = self.by_vertical_moment()
        for moment_1, moment_2 in \
            sequencetools.iterate_sequence_pairwise_strict(vertical_moments):
            for pair in sequencetools.yield_all_unordered_pairs_of_sequence(
                moment_1.start_leaves):
                yield pair
            pairs = sequencetools.yield_all_pairs_between_sequences(
                moment_1.leaves, moment_2.start_leaves)
            for pair in pairs:
                yield pair
        else:
            for pair in sequencetools.yield_all_unordered_pairs_of_sequence(
                moment_2.start_leaves):
                yield pair

    def by_logical_tie(
        self,
        nontrivial=False,
        pitched=False,
        reverse=False,
        ):
        r'''Iterate logical ties forward in `expr`:

        ::

            >>> staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16")

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                c'4 ~
                \times 2/3 {
                    c'16
                    d'8
                }
                e'8
                f'4 ~
                f'16
            }

        ::

            >>> for x in iterate(staff).by_logical_tie():
            ...     x
            ...
            LogicalTie(Note("c'4"), Note("c'16"))
            LogicalTie(Note("d'8"),)
            LogicalTie(Note("e'8"),)
            LogicalTie(Note("f'4"), Note("f'16"))

        Iterate logical ties backward in `expr`:

        ::

            >>> for x in iterate(staff).by_logical_tie(reverse=True):
            ...     x
            ...
            LogicalTie(Note("f'4"), Note("f'16"))
            LogicalTie(Note("e'8"),)
            LogicalTie(Note("d'8"),)
            LogicalTie(Note("c'4"), Note("c'16"))

        Iterate pitched logical ties in `expr`:

        ::

            >>> for x in iterate(staff).by_logical_tie(pitched=True):
            ...     x
            ...
            LogicalTie(Note("c'4"), Note("c'16"))
            LogicalTie(Note("d'8"),)
            LogicalTie(Note("e'8"),)
            LogicalTie(Note("f'4"), Note("f'16"))

        Iterate nontrivial logical ties in `expr`:

        ::

            >>> for x in iterate(staff).by_logical_tie(nontrivial=True):
            ...     x
            ...
            LogicalTie(Note("c'4"), Note("c'16"))
            LogicalTie(Note("f'4"), Note("f'16"))

        Returns generator.
        '''
        nontrivial = bool(nontrivial)
        prototype = scoretools.Leaf
        if pitched:
            prototype = (scoretools.Chord, scoretools.Note)
        if not reverse:
            for leaf in self.by_class(prototype):
                tie_spanners = leaf._get_spanners(spannertools.Tie)
                if not tie_spanners or \
                    tuple(tie_spanners)[0]._is_my_last_leaf(leaf):
                    logical_tie = leaf._get_logical_tie()
                    if not nontrivial or not logical_tie.is_trivial:
                        yield logical_tie
        else:
            for leaf in self.by_class(prototype, reverse=True):
                tie_spanners = leaf._get_spanners(spannertools.Tie)
                if not(tie_spanners) or \
                    tuple(tie_spanners)[0]._is_my_first_leaf(leaf):
                    logical_tie = leaf._get_logical_tie()
                    if not nontrivial or not logical_tie.is_trivial:
                        yield logical_tie

    def by_logical_voice(
        self,
        component_class,
        logical_voice,
        reverse=False,
        ):
        r'''Yield left-to-right instances of `component_class` in `expr`
        with `logical_voice`:

        ::

            >>> container_1 = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
            >>> container_1.is_simultaneous = True
            >>> container_1[0].name = 'voice 1'
            >>> container_1[1].name = 'voice 2'
            >>> container_2 = Container([Voice("g'8 a'8"), Voice("b'8 c''8")])
            >>> container_2.is_simultaneous = True
            >>> container_2[0].name = 'voice 1'
            >>> container_2[1].name = 'voice 2'
            >>> staff = Staff([container_1, container_2])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                <<
                    \context Voice = "voice 1" {
                        c'8
                        d'8
                    }
                    \context Voice = "voice 2" {
                        e'8
                        f'8
                    }
                >>
                <<
                    \context Voice = "voice 1" {
                        g'8
                        a'8
                    }
                    \context Voice = "voice 2" {
                        b'8
                        c''8
                    }
                >>
            }

        ::

            >>> leaf = staff.select_leaves(allow_discontiguous_leaves=True)[0]
            >>> signature = inspect(leaf).get_parentage().logical_voice
            >>> for x in iterate(staff).by_logical_voice(Note, signature):
            ...     x
            ...
            Note("c'8")
            Note("d'8")
            Note("g'8")
            Note("a'8")

        Returns generator.
        '''
        if isinstance(self._client, component_class) and \
            self._client._get_parentage().logical_voice == \
                logical_voice:
            yield self._client

        if not reverse:
            if isinstance(self._client, (list, tuple)):
                for component in self._client:
                    for x in iterate(component).by_logical_voice(
                        component_class,
                        logical_voice,
                        ):
                        yield x
            if hasattr(self._client, '_music'):
                for component in self._client._music:
                    for x in iterate(component).by_logical_voice(
                        component_class,
                        logical_voice,
                        ):
                        yield x
        else:
            if isinstance(self._client, (list, tuple)):
                for component in reversed(self._client):
                    for x in iterate(component).by_logical_voice(
                        component_class,
                        logical_voice,
                        reverse=True,
                        ):
                        yield x
            if hasattr(self._client, '_music'):
                for component in reversed(self._client._music):
                    for x in iterate(component).by_logical_voice(
                        component_class,
                        logical_voice,
                        reverse=True,
                        ):
                        yield x

    def by_logical_voice_from_component(
        self,
        component_class=None,
        reverse=False,
        ):
        r'''Iterate logical voice forward from `component` and yield instances
        of `component_class`.

        ::

            >>> container_1 = Container([Voice("c'8 d'8"), Voice("e'8 f'8")])
            >>> container_1.is_simultaneous = True
            >>> container_1[0].name = 'voice 1'
            >>> container_1[1].name = 'voice 2'
            >>> container_2 = Container([Voice("g'8 a'8"), Voice("b'8 c''8")])
            >>> container_2.is_simultaneous = True
            >>> container_2[0].name = 'voice 1'
            >>> container_2[1].name = 'voice 2'
            >>> staff = Staff([container_1, container_2])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                <<
                    \context Voice = "voice 1" {
                        c'8
                        d'8
                    }
                    \context Voice = "voice 2" {
                        e'8
                        f'8
                    }
                >>
                <<
                    \context Voice = "voice 1" {
                        g'8
                        a'8
                    }
                    \context Voice = "voice 2" {
                        b'8
                        c''8
                    }
                >>
            }

        Starting from the first leaf in score:

        ::

            >>> leaf = staff.select_leaves(allow_discontiguous_leaves=True)[0]
            >>> for x in iterate(leaf).by_logical_voice_from_component(Note):
            ...     x
            ...
            Note("c'8")
            Note("d'8")
            Note("g'8")
            Note("a'8")

        Starting from the second leaf in score:

        ::

            >>> leaf = staff.select_leaves(allow_discontiguous_leaves=True)[1]
            >>> for x in iterate(leaf).by_logical_voice_from_component(Note):
            ...     x
            ...
            Note("d'8")
            Note("g'8")
            Note("a'8")

        Yield all components in logical voice:

        ::

            >>> leaf = staff.select_leaves(allow_discontiguous_leaves=True)[0]
            >>> for x in iterate(leaf).by_logical_voice_from_component():
            ...     x
            ...
            Note("c'8")
            Voice-"voice 1"{2}
            Note("d'8")
            Voice-"voice 1"{2}
            Note("g'8")
            Note("a'8")

        Iterate logical voice backward from `component` and yield instances
        of `component_class`, starting from the last leaf in score:

        ::

            >>> leaf = staff.select_leaves(allow_discontiguous_leaves=True)[-1]
            >>> for x in iterate(leaf).by_logical_voice_from_component(
            ...     Note,
            ...     reverse=True,
            ...     ):
            ...     x
            Note("c''8")
            Note("b'8")
            Note("f'8")
            Note("e'8")

        Yield all components in logical voice:

        ::

            >>> leaf = staff.select_leaves(allow_discontiguous_leaves=True)[-1]
            >>> for x in iterate(leaf).by_logical_voice_from_component(
            ...     reverse=True,
            ...     ):
            ...     x
            Note("c''8")
            Voice-"voice 2"{2}
            Note("b'8")
            Voice-"voice 2"{2}
            Note("f'8")
            Note("e'8")

        Returns generator.
        '''
        # set default class
        if component_class is None:
            component_class = scoretools.Component
        # save logical voice signature of input component
        signature = self._client._get_parentage().logical_voice
        # iterate component depth-first allowing to crawl UP into score
        if not reverse:
            for x in iterate(self._client).depth_first(
                capped=False):
                if isinstance(x, component_class):
                    if x._get_parentage().logical_voice == signature:
                        yield x
        else:
            for x in iterate(self._client).depth_first(
                capped=False, direction=Right):
                if isinstance(x, component_class):
                    if x._get_parentage().logical_voice == signature:
                        yield x

    def by_run(self, classes):
        r'''Iterate runs in expression.

        ..  container:: example

            **Example 1.** Iterate runs of notes and chords at only the
            top level of score:

            ::

                >>> staff = Staff(r"\times 2/3 { c'8 d'8 r8 }")
                >>> staff.append(r"\times 2/3 { r8 <e' g'>8 <f' a'>8 }")
                >>> staff.extend("g'8 a'8 r8 r8 <b' d''>8 <c'' e''>8")

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 2/3 {
                        c'8
                        d'8
                        r8
                    }
                    \times 2/3 {
                        r8
                        <e' g'>8
                        <f' a'>8
                    }
                    g'8
                    a'8
                    r8
                    r8
                    <b' d''>8
                    <c'' e''>8
                }

            ::

                >>> for group in iterate(staff[:]).by_run((Note, Chord)):
                ...     group
                ...
                (Note("g'8"), Note("a'8"))
                (Chord("<b' d''>8"), Chord("<c'' e''>8"))

        ..  container:: example

            **Example 2.** Iterate runs of notes and chords at all levels of
            score:

            ::

                >>> leaves = iterate(staff).by_class(scoretools.Leaf)

            ::

                >>> for group in iterate(leaves).by_run((Note, Chord)):
                ...     group
                ...
                (Note("c'8"), Note("d'8"))
                (Chord("<e' g'>8"), Chord("<f' a'>8"), Note("g'8"), Note("a'8"))
                (Chord("<b' d''>8"), Chord("<c'' e''>8"))

        Returns generator.
        '''
        from abjad.tools import selectiontools
        sequence = selectiontools.SliceSelection(self._client)
        current_group = ()
        for group in sequence.group_by(type):
            if type(group[0]) in classes:
                current_group = current_group + group
            elif current_group:
                yield current_group
                current_group = ()
        if current_group:
            yield current_group

    def by_semantic_voice(
        self,
        reverse=False,
        start=0,
        stop=None,
        ):
        r'''Iterate semantic voices forward in `expr`:

        ::

            >>> measures = scoretools.make_spacer_skip_measures(
            ...     [(3, 8), (5, 16), (5, 16)])
            >>> time_signature_voice = Voice(measures)
            >>> time_signature_voice.name = 'TimeSignatuerVoice'
            >>> time_signature_voice.is_nonsemantic = True
            >>> music_voice = Voice("c'4. d'4 e'16 f'4 g'16")
            >>> music_voice.name = 'MusicVoice'
            >>> staff = Staff([time_signature_voice, music_voice])
            >>> staff.is_simultaneous = True

        ..  doctest::

            >>> print format(staff)
            \new Staff <<
                \context Voice = "TimeSignatuerVoice" {
                    {
                        \time 3/8
                        s1 * 3/8
                    }
                    {
                        \time 5/16
                        s1 * 5/16
                    }
                    {
                        s1 * 5/16
                    }
                }
                \context Voice = "MusicVoice" {
                    c'4.
                    d'4
                    e'16
                    f'4
                    g'16
                }
            >>

            >>> for voice in iterate(staff).by_semantic_voice():
            ...   voice
            ...
            Voice-"MusicVoice"{5}

        Iterate semantic voices backward in `expr`:

        ::

            >>> for voice in iterate(staff).by_semantic_voice(reverse=True):
            ...   voice
            ...
            Voice-"MusicVoice"{5}

        Returns generator.
        '''
        for voice in self.by_class(
            scoretools.Voice,
            reverse=reverse,
            start=start,
            stop=stop,
            ):
            if not voice.is_nonsemantic:
                yield voice

    def by_timeline(
        self,
        component_class=None,
        reverse=False,
        ):
        r'''Iterate timeline forward in `expr`:

        ::

            >>> score = Score([])
            >>> score.append(Staff("c'4 d'4 e'4 f'4"))
            >>> score.append(Staff("g'8 a'8 b'8 c''8"))

        ..  doctest::

            >>> print format(score)
            \new Score <<
                \new Staff {
                    c'4
                    d'4
                    e'4
                    f'4
                }
                \new Staff {
                    g'8
                    a'8
                    b'8
                    c''8
                }
            >>

        ::

            >>> for leaf in iterate(score).by_timeline():
            ...     leaf
            ...
            Note("c'4")
            Note("g'8")
            Note("a'8")
            Note("d'4")
            Note("b'8")
            Note("c''8")
            Note("e'4")
            Note("f'4")

        Iterate timeline backward in `expr`:

        ::

            >>> for leaf in iterate(score).by_timeline(reverse=True):
            ...     leaf
            ...
            Note("f'4")
            Note("e'4")
            Note("d'4")
            Note("c''8")
            Note("b'8")
            Note("c'4")
            Note("a'8")
            Note("g'8")

        Iterate leaves when `component_class` is none.

        .. todo:: optimize to avoid behind-the-scenes full-score traversal.
        '''
        if component_class is None:
            component_class = scoretools.Leaf
        component_generator = iterate(self._client).by_class(component_class)
        components = list(component_generator)
        def _backward_sort_helper(component_1, component_2):
            result = cmp(
                component_1._get_timespan().stop_offset,
                component_2._get_timespan().stop_offset)
            if result == 0:
                return cmp(
                    component_1._get_parentage().score_index,
                    component_2._get_parentage().score_index)
            else:
                # note negative result of cmp() is returned
                # for backward time sort
                return -result
        def _forward_sort_helper(component_1, component_2):
            result = cmp(
                component_1._get_timespan().start_offset,
                component_2._get_timespan().start_offset)
            if result == 0:
                return cmp(
                    component_1._get_parentage().score_index,
                    component_2._get_parentage().score_index)
            else:
                return result
        if not reverse:
            components.sort(_forward_sort_helper)
        else:
            components.sort(_backward_sort_helper)
        for component in components:
            yield component

    def by_timeline_from_component(
        self,
        component_class=None,
        reverse=False,
        ):
        r'''Iterate timeline forward from `component`:

        ::

            >>> score = Score([])
            >>> score.append(Staff("c'4 d'4 e'4 f'4"))
            >>> score.append(Staff("g'8 a'8 b'8 c''8"))

        ..  doctest::

            >>> print format(score)
            \new Score <<
                \new Staff {
                    c'4
                    d'4
                    e'4
                    f'4
                }
                \new Staff {
                    g'8
                    a'8
                    b'8
                    c''8
                }
            >>

        ::

            >>> for leaf in iterate(score[1][2]).by_timeline_from_component():
            ...     leaf
            ...
            Note("b'8")
            Note("c''8")
            Note("e'4")
            Note("f'4")

        Iterate timeline backward from `component`:

        ::

        ::

            >>> for leaf in iterate(score[1][2]).by_timeline_from_component(
            ...     reverse=True):
            ...     leaf
            ...
            Note("b'8")
            Note("c'4")
            Note("a'8")
            Note("g'8")

        Yield components sorted backward by score offset stop time
        when `reverse` is True.

        Iterate leaves when `component_class` is none.

        .. todo:: optimize to avoid behind-the-scenes full-score traversal.
        '''
        assert isinstance(self._client, scoretools.Component)
        if component_class is None:
            component_class = scoretools.Leaf
        root = self._client._get_parentage().root
        component_generator = iterate(root).by_timeline(
            component_class=component_class,
            reverse=reverse,
            )
        yielded_expr = False
        for component in component_generator:
            if yielded_expr:
                yield component
            elif component is self._client:
                yield component
                yielded_expr = True

    def by_topmost_logical_ties_and_components(self):
        r'''Iterate topmost logical ties and components forward in `expr`:

        ::

            >>> string = r"c'8 ~ c'32 d'8 ~ d'32 \times 2/3 { e'8 f'8 g'8 } "
            >>> string += "a'8 ~ a'32 b'8 ~ b'32"
            >>> staff = Staff(string)

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                c'8 ~
                c'32
                d'8 ~
                d'32
                \times 2/3 {
                    e'8
                    f'8
                    g'8
                }
                a'8 ~
                a'32
                b'8 ~
                b'32
            }

        ::

            >>> for x in iterate(staff).by_topmost_logical_ties_and_components():
            ...     x
            ...
            LogicalTie(Note("c'8"), Note("c'32"))
            LogicalTie(Note("d'8"), Note("d'32"))
            Tuplet(Multiplier(2, 3), "e'8 f'8 g'8")
            LogicalTie(Note("a'8"), Note("a'32"))
            LogicalTie(Note("b'8"), Note("b'32"))

        Raise logical tie error on overlapping logical ties.

        Returns generator.
        '''
        from abjad.tools import selectiontools
        prototype = (spannertools.Tie,)
        if isinstance(self._client, scoretools.Leaf):
            logical_tie = self._client._get_logical_tie()
            if len(logical_tie) == 1:
                yield logical_tie
            else:
                message = 'can not have only one leaf in logical tie.'
                raise ValueError(message)
        elif isinstance(
            self._client, (
                collections.Sequence,
                scoretools.Container,
                selectiontools.SliceSelection,
                )):
            for component in self._client:
                if isinstance(component, scoretools.Leaf):
                    tie_spanners = component._get_spanners(prototype)
                    if not tie_spanners or \
                        tuple(tie_spanners)[0]._is_my_last_leaf(component):
                        yield component._get_logical_tie()
                elif isinstance(component, scoretools.Container):
                    yield component
        else:
            message = 'input must be iterable: {!r}.'.format(self._client)
            raise ValueError(message)

    def by_vertical_moment(
        self,
        reverse=False,
        ):
        r'''Iterate vertical moments forward in `expr`:

        ::

            >>> score = Score([])
            >>> staff = Staff(r"\times 4/3 { d''8 c''8 b'8 }")
            >>> score.append(staff)

        ::

            >>> piano_staff = scoretools.PianoStaff([])
            >>> piano_staff.append(Staff("a'4 g'4"))
            >>> piano_staff.append(Staff(r"""\clef "bass" f'8 e'8 d'8 c'8"""))
            >>> score.append(piano_staff)

        ..  doctest::

            >>> print format(score)
            \new Score <<
                \new Staff {
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 4/3 {
                        d''8
                        c''8
                        b'8
                    }
                }
                \new PianoStaff <<
                    \new Staff {
                        a'4
                        g'4
                    }
                    \new Staff {
                        \clef "bass"
                        f'8
                        e'8
                        d'8
                        c'8
                    }
                >>
            >>

        ::

            >>> for x in iterate(score).by_vertical_moment():
            ...     x.leaves
            ...
            (Note("d''8"), Note("a'4"), Note("f'8"))
            (Note("d''8"), Note("a'4"), Note("e'8"))
            (Note("c''8"), Note("a'4"), Note("e'8"))
            (Note("c''8"), Note("g'4"), Note("d'8"))
            (Note("b'8"), Note("g'4"), Note("d'8"))
            (Note("b'8"), Note("g'4"), Note("c'8"))

        ::

            >>> for x in iterate(piano_staff).by_vertical_moment():
            ...     x.leaves
            ...
            (Note("a'4"), Note("f'8"))
            (Note("a'4"), Note("e'8"))
            (Note("g'4"), Note("d'8"))
            (Note("g'4"), Note("c'8"))

        Iterate vertical moments backward in `expr`:

        ::

        ::

            >>> for x in iterate(score).by_vertical_moment(reverse=True):
            ...     x.leaves
            ...
            (Note("b'8"), Note("g'4"), Note("c'8"))
            (Note("b'8"), Note("g'4"), Note("d'8"))
            (Note("c''8"), Note("g'4"), Note("d'8"))
            (Note("c''8"), Note("a'4"), Note("e'8"))
            (Note("d''8"), Note("a'4"), Note("e'8"))
            (Note("d''8"), Note("a'4"), Note("f'8"))

        ::

            >>> for x in iterate(piano_staff).by_vertical_moment(
            ...     reverse=True):
            ...     x.leaves
            ...
            (Note("g'4"), Note("c'8"))
            (Note("g'4"), Note("d'8"))
            (Note("a'4"), Note("e'8"))
            (Note("a'4"), Note("f'8"))

        Returns generator.
        '''
        from abjad.tools import selectiontools

        def _buffer_components_starting_with(component, buffer, stop_offsets):
            #if not isinstance(component, scoretools.Component):
            #    raise TypeError
            buffer.append(component)
            stop_offsets.append(component._get_timespan().stop_offset)
            if isinstance(component, scoretools.Container):
                if component.is_simultaneous:
                    for x in component:
                        _buffer_components_starting_with(
                            x, buffer, stop_offsets)
                else:
                    if component:
                        _buffer_components_starting_with(
                            component[0], buffer, stop_offsets)

        def _iterate_vertical_moments_forward_in_expr(expr):
            #if not isinstance(expr, scoretools.Component):
            #    raise TypeError
            governors = (expr,)
            current_offset, stop_offsets, buffer = \
                durationtools.Offset(0), [], []
            _buffer_components_starting_with(expr, buffer, stop_offsets)
            while buffer:
                vertical_moment = selectiontools.VerticalMoment()
                offset = durationtools.Offset(current_offset)
                components = list(buffer)
                components.sort(key=lambda x: x._get_parentage().score_index)
                vertical_moment._offset = offset
                vertical_moment._governors = governors
                vertical_moment._components = components
                yield vertical_moment
                current_offset, stop_offsets = min(stop_offsets), []
                _update_buffer(current_offset, buffer, stop_offsets)

        def _next_in_parent(component):
            from abjad.tools import selectiontools
            if not isinstance(component, scoretools.Component):
                raise TypeError
            selection = selectiontools.SliceSelection(component)
            parent, start, stop = \
                selection._get_parent_and_start_stop_indices()
            assert start == stop
            if parent is None:
                raise StopIteration
            # can not advance within simultaneous parent
            if parent.is_simultaneous:
                raise StopIteration
            try:
                return parent[start + 1]
            except IndexError:
                raise StopIteration

        def _update_buffer(current_offset, buffer, stop_offsets):
            #print 'At %s with %s ...' % (current_offset, buffer)
            for component in buffer[:]:
                if component._get_timespan().stop_offset <= current_offset:
                    buffer.remove(component)
                    try:
                        next_component = _next_in_parent(component)
                        _buffer_components_starting_with(
                            next_component, buffer, stop_offsets)
                    except StopIteration:
                        pass
                else:
                    stop_offsets.append(component._get_timespan().stop_offset)

        if not reverse:
            for x in _iterate_vertical_moments_forward_in_expr(self._client):
                yield x
        else:
            moments_in_governor = []
            for component in self.by_class():
                offset = component._get_timespan().start_offset
                if offset not in moments_in_governor:
                    moments_in_governor.append(offset)
            moments_in_governor.sort()
            for moment_in_governor in reversed(moments_in_governor):
                yield self._client._get_vertical_moment_at(moment_in_governor)

    def depth_first(
        self,
        capped=True,
        direction=Left,
        forbid=None,
        unique=True,
        ):
        '''Iterate components depth-first from `component`.

        .. todo:: Add usage examples.
        '''

        def _next_node_depth_first(component, total):
            r'''If client has unvisited music, return next unvisited node in
            client's music.

            If client has no univisited music and has a parent, return client's
            parent.

            If client has no univisited music and no parent, return None.
            '''

            client = component
            if hasattr(client, '_music') and \
                0 < len(client) and \
                total < len(client):
                return client[total], 0
            else:
                parent = client._parent
                if parent is not None:
                    return parent, parent.index(client) + 1
                else:
                    return None, None

        def _previous_node_depth_first(component, total=0):
            r'''If client has unvisited music, return previous unvisited node in
            client's music.

            If client has no univisited music and has a parent, return client's
            parent.

            If client has no univisited music and no parent, return None.
            '''

            client = component
            if hasattr(client, '_music') and \
                0 < len(client) and \
                total < len(client):
                return client[len(client) - 1 - total], 0
            else:
                parent = client._parent
                if parent is not None:
                    return parent, len(parent) - parent.index(client)
                else:
                    return None, None

        def _handle_forbidden_node(node, queue):
            node_parent = node._parent
            if node_parent is not None:
                rank = node_parent.index(node) + 1
                node = node_parent
            else:
                node, rank = None, None
            queue.pop()
            return node, rank

        def _advance_node_depth_first(node, rank, direction):
            # TODO: remove 'left'
            if direction in ('left', Left):
                node, rank = _next_node_depth_first(node, rank)
            else:
                node, rank = _previous_node_depth_first(node, rank)
            return node, rank

        def _is_node_forbidden(node, forbid):
            if forbid is None:
                return False
            elif forbid == 'simultaneous':
                return getattr(node, 'is_simultaneous', False)
            else:
                return isinstance(node, forbid)

        def _find_yield(node, rank, queue, unique):
            if hasattr(node, '_music'):
                try:
                    visited = node is queue[-1]
                except IndexError:
                    visited = False
                if not visited or unique is not True:
                    queue.append(node)
                    return node
                elif rank == len(node):
                    queue.pop()
                    return None
            else:
                return node

        assert isinstance(self._client, scoretools.Component)
        component = self._client
        client_parent, node, rank = component._parent, component, 0
        queue = collections.deque([])
        while node is not None and not (capped and node is client_parent):
            result = _find_yield(node, rank, queue, unique)
            if result is not None:
                yield result
            if _is_node_forbidden(node, forbid):
                node, rank = _handle_forbidden_node(node, queue)
            else:
                node, rank = _advance_node_depth_first(node, rank, direction)
        queue.clear()



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_LayoutBlock_lilypond_command_01():

    staff = Staff("fs'2 fs'2 gs'2 gs'2")
    score = Score([staff])
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    lilypond_file.layout_block.append(indicatortools.LilyPondCommand('accidentalStyle forget'))

    assert systemtools.TestManager.compare(
        lilypond_file.layout_block,
        r'''
        \layout {
            \accidentalStyle forget
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.spannertools.Beam import Beam


class ComplexBeam(Beam):
    r'''A complex beam spanner.

    ::

        >>> staff = Staff("c'16 e'16 r16 f'16 g'2")
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'16
            e'16
            r16
            f'16
            g'2
        }

    ::

        >>> beam = spannertools.ComplexBeam()
        >>> attach(beam, staff[:4])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            e'16 ]
            r16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 [ ]
            g'2
        }

    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None, 
        lone=False, 
        direction=None,
        overrides=None,
        ):
        Beam.__init__(
            self, 
            components=components, 
            direction=direction,
            overrides=overrides,
            )
        self.lone = lone

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        Beam._copy_keyword_args(self, new)
        new.lone = self.lone

    def _format_before_leaf(self, leaf):
        r'''Spanner format contribution to output before leaf.
        '''
        result = []
        result.extend(Beam._format_before_leaf(self, leaf))
        if self.is_beamable_component(leaf):
            if self._is_my_only_leaf(leaf):
                left, right = self._get_left_right_for_lone_leaf(leaf)
            elif self._is_exterior_leaf(leaf):
                left, right = self._get_left_right_for_exterior_leaf(leaf)
            else:
                left, right = self._get_left_right_for_interior_leaf(leaf)
            if left is not None:
                result.append(r'\set stemLeftBeamCount = #%s' % left)
            if right is not None:
                result.append(r'\set stemRightBeamCount = #%s' % right)
        return result

    def _format_right_of_leaf(self, leaf):
        r'''Spanner format contribution to output right of leaf.
        '''
        from abjad.tools import scoretools
        result = []
        #if leaf.beam.beamable:
        if self.is_beamable_component(leaf):
            previous_leaf = leaf._get_leaf(-1)
            next_leaf = leaf._get_leaf(1)
            # lone
            if self._is_my_only_leaf(leaf):
                if self.lone:
                    if self.direction is not None:
                        result.append('%s [' % self.direction)
                    else:
                        result.append('[')
            # otherwise
            elif self._is_my_first_leaf(leaf) or not previous_leaf or \
                not self.is_beamable_component(previous_leaf):
                if self.direction is not None:
                    result.append('%s [' % self.direction)
                else:
                    result.append('[')
            # lone
            if self._is_my_only_leaf(leaf):
                if self.lone:
                    result.append(']')
            # otherwise
            elif self._is_my_last_leaf(leaf) or not next_leaf or \
                not self.is_beamable_component(next_leaf):
                result.append(']')
        return result

    def _get_left_right_for_exterior_leaf(self, leaf):
        r'''Get left and right flag counts for exterior leaf in spanner.
        '''
        from abjad.tools import scoretools
        # lone
        if self._is_my_only_leaf(leaf):
            left, right = self._get_left_right_for_lone_leaf(leaf)
        # first
        elif self._is_my_first_leaf(leaf) or not leaf._get_leaf(-1):
            left = 0
            right = leaf.written_duration.flag_count
        # last
        elif self._is_my_last_leaf(leaf) or leaf._get_leaf(1):
            left = leaf.written_duration.flag_count
            right = 0
        else:
            message = 'leaf must be first or last in spanner.'
            raise ValueError(message)
        return left, right

    def _get_left_right_for_interior_leaf(self, leaf):
        r'''Interior leaves are neither first nor last in spanner.
        Interior leaves may be surrounded by beamable leaves.
        Interior leaves may be surrounded by unbeamable leaves.
        Four cases total for beamability of surrounding leaves.
        '''
        from abjad.tools import scoretools
        previous_leaf = leaf._get_leaf(-1)
        previous_written = previous_leaf.written_duration
        current_written = leaf.written_duration
        next_leaf = leaf._get_leaf(1)
        next_written = next_leaf.written_duration
        previous_flag_count = previous_written.flag_count
        current_flag_count = current_written.flag_count
        next_flag_count = next_written.flag_count
        # [unbeamable leaf beamable]
        if not self.is_beamable_component(previous_leaf) and \
            self.is_beamable_component(next_leaf):
            left = current_flag_count
            right = min(current_flag_count, next_flag_count)
        # [beamable leaf unbeamable]
        if self.is_beamable_component(previous_leaf) and \
            not self.is_beamable_component(next_leaf):
            left = min(current_flag_count, previous_flag_count)
            right = current_flag_count
        # [unbeamable leaf unbeamable]
        elif not self.is_beamable_component(previous_leaf) and \
            not self.is_beamable_component(next_leaf):
            left = current_flag_count
            right = current_flag_count
        # [beamable leaf beamable]
        else:
            left = min(current_flag_count, previous_flag_count)
            right = min(current_flag_count, next_flag_count)
            if left != current_flag_count and right != current_flag_count:
                left = current_flag_count
        return left, right

    def _get_left_right_for_lone_leaf(self, leaf):
        r'''Get left and right flag counts for only leaf in spanner.
        '''
        current_flag_count = leaf.written_duration.flag_count
        left, right = None, None
        if self.lone == 'left':
            left = current_flag_count
            right = 0
        elif self.lone == 'right':
            left = 0
            right = current_flag_count
        elif self.lone in ('both', True):
            left = current_flag_count
            right = current_flag_count
        elif self.lone in ('neither', False):
            left = None
            right = None
        else:
            message = 'long must be left, right or both: {!r}.'.format(lone)
            raise ValueError(message)
        return left, right

    ### PUBLIC PROPERTIES ###

    @property
    def lone(self):
        r'''Beam lone leaf and force beam nibs to left:

        ::

            >>> note = Note("c'16")

        ::

            >>> beam = spannertools.ComplexBeam(lone='left')
            >>> attach(beam, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print format(note)
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            c'16 [ ]

        Beam lone leaf and force beam nibs to right:

        ::

            >>> note = Note("c'16")

        ::

            >>> beam = spannertools.ComplexBeam(lone='right')
            >>> attach(beam, note)

        ..  doctest::

            >>> print format(note)
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [ ]

        Beam lone leaf and force beam nibs to both left and right:

        ::

            >>> note = Note("c'16")

        ::

            >>> beam = spannertools.ComplexBeam(lone='both')
            >>> attach(beam, note)

        ..  doctest::

            >>> print format(note)
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            c'16 [ ]

        Beam lone leaf and accept LilyPond default nibs at 
        both left and right:

        ::

            >>> note = Note("c'16")

        ::

            >>> beam = spannertools.ComplexBeam(lone=True)
            >>> attach(beam, note)

        ..  doctest::

            >>> print format(note)
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            c'16 [ ]

        Do not beam lone leaf:

        ::

            >>> note = Note("c'16")

        ::

            >>> beam = spannertools.ComplexBeam(lone=False)
            >>> attach(beam, note)

        ..  doctest::

            >>> print format(note)
            c'16

        Set to ``'left'``, ``'right'``, ``'both'``, true or false 
        as shown above.

        Ignore this setting when spanner contains more than one leaf.
        '''
        return self._lone

    @lone.setter
    def lone(self, arg):
        assert isinstance(arg, bool) or arg in ('left', 'right', 'both')
        self._lone = arg
